/************************************************************************
                  		CAN BUS EXAMPLE - NODE: COLLECTOR
                  		================================

This is the COLLECTOR node of the CAN bus example. In this project a PIC18F258 type
microcontroller is used. An MCP2551 type CAN bus transceiver is used to connect
the microcontroller to the CAN bus. The microcontroller is operated from an 8MHz
crystal with an external reset button.

Pin CANRX and CANTX of the microcontroller are connected to pins RXD and TXD of the
transceiver chip respectively. Pins CANH and CANL of the transceiver chip are connected to
the CAN bus.

An LM35DZ type analog temperature sensor is connected to port AN0 of the microcontroller. 
The microcontroller reads the temperature when a request is received and then sends the temperature value as a byte to Node:DISPLAY on the CAN bus.

CAN speed parameters are:

    Microcontroller clock:  	8MHz
    CAN Bus bit rate:       		100Kb/s
    Sync_Seg:               	1
    Prop_Seg:               	6
    Phase_Seg1:             	6
    Phase_Seg2:             	7
    SJW:                    	1
    BRP:                    	1
    Sample point:           	65%

Author:  Dogan Ibrahim
Date:    October 2013
File:    CAN-COLLECTOR.C
**************************************************************/

void main()
{
     unsigned char temperature, dat[8];
     unsigned short init_flag, send_flag, dt, len, read_flag;
     char SJW, BRP, Phase_Seg1, Phase_Seg2, Prop_Seg, txt[4];
     unsigned int temp;
     unsigned long mV;
     long id, mask;

     TRISA = 0xFF;                      			// PORT A are inputs
     TRISB = 0x08;                      			// RB2 is output, RB3 is input
//
// Configure A/D converter
//
     ADCON1 = 0x80;
//
// CAN BUS Timing Parameters
//
     SJW = 1;
     BRP = 1;
     Phase_Seg1 = 6;
     Phase_Seg2 = 7;
     BRP = 1;
     Prop_Seg = 6;

     init_flag = _CAN_CONFIG_SAMPLE_THRICE     &
                        _CAN_CONFIG_PHSEG2_PRG_ON  &
                        _CAN_CONFIG_STD_MSG                &
                        _CAN_CONFIG_DBL_BUFFER_ON  &
                        _CAN_CONFIG_VALID_XTD_MSG  &
                        _CAN_CONFIG_LINE_FILTER_OFF;

     send_flag = _CAN_TX_PRIORITY_0             &
                           _CAN_TX_XTD_FRAME           &
                           _CAN_TX_NO_RTR_FRAME;

     read_flag = 0;
//
// Initialise CAN module
//
      CANInitialize(SJW, BRP, Phase_Seg1, Phase_Seg2, Prop_Seg, init_flag);
//
// Set CAN CONFIG mode
//
      CANSetOperationMode(_CAN_MODE_CONFIG, 0xFF);

      mask = -1;
//
// Set all MASK1 bits to 1's
//
      CANSetMask(_CAN_MASK_B1, mask, _CAN_CONFIG_XTD_MSG);
//
// Set all MASK2 bits to 1's
//
      CANSetMask(_CAN_MASK_B2, mask, _CAN_CONFIG_XTD_MSG);
//
// Set id of filter B2_F3 to 500
//
      CANSetFilter(_CAN_FILTER_B2_F3,500,_CAN_CONFIG_XTD_MSG);
//
// Set CAN module to NORMAL mode
//
      CANSetOperationMode(_CAN_MODE_NORMAL, 0xFF);

//
// Program loop. Read the temperature from analog temperature
// sensor
//
     for(;;)                                      			// Endless loop
     {
         //
         // Wait until a request is received
         //
         dt = 0;
         while(!dt) dt = CANRead(&id, dat, &len, &read_flag);
         if(id == 500 && dat[0] == 'T')
         {
               //
               // Now read the temperature
               //
               temp = Adc_Read(0);                            		// Read temp
               mV = (unsigned long)temp * 5000 / 1024;        		// in mV
               temperature = mV/10;                           		// in degrees C
               //
               // send the temperature to Node:Display
               //
               dat[0] = temperature;
               id = 3;                                    		// Identifier
               CANWrite(id, dat, 1, send_flag);          		// Send temperature
         }
      }
}

