/************************************************************************
                  		CAN BUS EXAMPLE - NODE: DISPLAY
                  		==============================

This is the DISPLAY node of the CAN bus example. In this project a PIC18F258 type
microcontroller is used. An MCP2551 type CAN bus transceiver is used to connect the
microcontroller to the CAN bus. The microcontroller is operated from an 8MHz
crystal with an external reset button.

Pin CANRX and CANTX of the microcontroller are connected to pins RXD and TXD of the
transceiver chip respectively. Pins CANH and CANL of the transceiver chip are connected
to the CAN bus.

An LCD is connected to PORT C of the microcontroller. The ambient temperature is read from
another CAN node and is displayed on the LCD.

The LCD is connected to the microcontroller as follows:

Microcontroller      	LCD

    RC0              	D4
    RC1              	D5
    RC2              	D6
    RC3              	D
    RC4              	RS
    RC5              	EN

CAN speed parameters are:

    Microcontroller clock:  	8MHz
    CAN Bus bit rate:       	100Kb/s
    Sync_Seg:               	1
    Prop_Seg:               	6
    Phase_Seg1:             	6
    Phase_Seg2:             	7
    SJW:                    	1
    BRP:                    	1
    Sample point:           	65%

Author:   Dogan Ibrahim
Date:     October 2013
File:     CAN-DISPLAY.C
**************************************************************/
// LCD module connections
sbit LCD_RS at RC4_bit;
sbit LCD_EN at RC5_bit;
sbit LCD_D4 at RC0_bit;
sbit LCD_D5 at RC1_bit;
sbit LCD_D6 at RC2_bit;
sbit LCD_D7 at RC3_bit;

sbit LCD_RS_Direction at TRISC4_bit;
sbit LCD_EN_Direction at TRISC5_bit;
sbit LCD_D4_Direction at TRISC0_bit;
sbit LCD_D5_Direction at TRISC1_bit;
sbit LCD_D6_Direction at TRISC2_bit;
sbit LCD_D7_Direction at TRISC3_bit;
// End LCD module connections

void main()
{
     unsigned char temperature, dat[8];
     unsigned char init_flag, send_flag, dt, len, read_flag;
     char SJW, BRP, Phase_Seg1, Phase_Seg2, Prop_Seg, txt[4];
     long id, mask;

     TRISC = 0;                         			// PORTC are outputs (LCD)
     TRISB = 0x08;                      			// RB2 is output, RB3 is input
//
// CAN BUS Parameters
//
     SJW = 1;
     BRP = 1;
     Phase_Seg1 = 6;
     Phase_Seg2 = 7;
     Prop_Seg = 6;

     init_flag = _CAN_CONFIG_SAMPLE_THRICE    &
                        _CAN_CONFIG_PHSEG2_PRG_ON  &
                        _CAN_CONFIG_STD_MSG                &
                        _CAN_CONFIG_DBL_BUFFER_ON  &
                        _CAN_CONFIG_VALID_XTD_MSG  &
                        _CAN_CONFIG_LINE_FILTER_OFF;

     send_flag = _CAN_TX_PRIORITY_0            &
                          _CAN_TX_XTD_FRAME           &
                          _CAN_TX_NO_RTR_FRAME;

     read_flag = 0;
//
// Initialise CAN module
//
      CANInitialize(SJW, BRP, Phase_Seg1, Phase_Seg2, Prop_Seg, init_flag);
//
// Set CAN CONFIG mode
//
      CANSetOperationMode(_CAN_MODE_CONFIG, 0xFF);

      mask = -1;
//
// Set all MASK1 bits to 1's
//
      CANSetMask(_CAN_MASK_B1, mask, _CAN_CONFIG_XTD_MSG);
//
// Set all MASK2 bits to 1's
//
      CANSetMask(_CAN_MASK_B2, mask, _CAN_CONFIG_XTD_MSG);
//
// Set id of filter B2_F3 to 3
//
      CANSetFilter(_CAN_FILTER_B2_F3,3,_CAN_CONFIG_XTD_MSG);
//
// Set CAN module to NORMAL mode
//
      CANSetOperationMode(_CAN_MODE_NORMAL, 0xFF);

//
// Configure LCD
//
     Lcd_Init();    		                  		// Initializd LCD
     Lcd_Cmd(_LCD_CLEAR);                			// Clear LCD
     Lcd_Out(1,1,"CAN BUS");            			// Display heading on LCD
     Delay_ms(1000);                    			// Wait for 2 seconds

//
// Program loop. Read the temperature from Node:COLLECTOR and display
// on the LCD continuously
//
     for(;;)                                        		// Endless loop
     {
         Lcd_Cmd(_LCD_CLEAR);                        		// Clear LCD
         Lcd_Out(1,1,"Temp = ");                    		// Display "Temp = "
         //
         // Send a message to Node:COLLECTOR and ask for data
         //
         dat[0] = 'T';                             		// Data to be sent
         id = 500;                                  		// Identifier
         CANWrite(id, dat, 1, send_flag);          		// Send 'T'
         //
         // Get temperature from node:COLLECT
         //
         dt = 0;
         while(!dt)dt = CANRead(&id, dat, &len, &read_flag);
         if(id == 3)
         {
             temperature = dat[0];
             ByteToStr(temperature,txt);            		// Convert to string
             Lcd_Out(1,8,txt);                      		// Output to LCD
             Delay_ms(1000);                        		// Wait 1 second
         }
      }

}

