/*******************************************************************************
                                	BLUETOOTH COMMUNICATION
                                	==========================

This project is about using Bluetooth communication in a project. In this project a Bluetooth
module is used in slave mode. A PC is used in master mode. Messages sent to the slave module
are displayed on an LCD.

The Easy Bluetooth board (www.mikroe.com) is used in this project, connected to PORTC of an EasyPIC V7 development board. An LCD is connected to PORTB of the microcontroller as in the previous projects.

Author:   Dogan Ibrahim
Date:     September 2013
File:     MIKROC-BLUE1.C
*******************************************************************************/
// LCD module connections
sbit LCD_RS at RB4_bit;
sbit LCD_EN at RB5_bit;
sbit LCD_D4 at RB0_bit;
sbit LCD_D5 at RB1_bit;
sbit LCD_D6 at RB2_bit;
sbit LCD_D7 at RB3_bit;

sbit LCD_RS_Direction at TRISB4_bit;
sbit LCD_EN_Direction at TRISB5_bit;
sbit LCD_D4_Direction at TRISB0_bit;
sbit LCD_D5_Direction at TRISB1_bit;
sbit LCD_D6_Direction at TRISB2_bit;
sbit LCD_D7_Direction at TRISB3_bit;
// End LCD module connections

#define RESET PORTC.RC1

const CMD = 1;
const AOK = 2;
const CONN = 3;
const END = 4;

unsigned char Command_Mode, temp, Response, Data_Received;
unsigned char Buffer[6], Txt[16];
unsigned char Cnt = 0;
unsigned char i = 0;
//
// UART receive interrupt handler. In Command Mode we get the following responses:
// CMD, AOK, END, CONN. The interrupt handler returns responses in command mode
// and also stores the received data from the master
//
void interrupt(void)
{
  if(PIR1.RC1IF == 1)                 			// Is this a UART receive interrupt ?
  {
     temp = UART1_Read();           			// Read the received character
     if(Command_Mode == 1 && temp != 0x0)
     {
        Buffer[Cnt] = temp;
        Cnt++;
        if(Cnt == 4)
        {
          if(Buffer[0] == 'C' && Buffer[1] == 'O' && Buffer[2] == 'N' && Buffer[3] == 'N')
          {
             Response = CONN;
             Cnt = 0;
          }
        }
        
        if(Cnt == 5)
        {
            Cnt = 0;
            Response = 0;
            if(Buffer[0] == 'C' && Buffer[1] == 'M' && Buffer[2] == 'D' && Buffer[3] == 0x0D &&
                Buffer[4] == 0x0A)Response = CMD;
            if(Buffer[0] == 'A' && Buffer[1] == 'O' && Buffer[2] == 'K' && Buffer[3] == 0x0D &&
                Buffer[4] == 0x0A)Response = AOK;
            if(Buffer[0] == 'E' && Buffer[1] == 'N' && Buffer[2] == 'D' && Buffer[3] == 0x0D &&
                Buffer[4] == 0x0A)Response = END;
        }
     }
     else
     {
        if(temp == 0x0D)               			// If END of data
        {
           Data_Received = 1;          			// End of data received flag
           Txt[i] = 0x0;                		// Terminate data with NULL
        }
        else
        {
          Txt[i] = temp;             			// Store received data
          i++;                       			// Increment for next character
        }
     }
     PIR1.RC1IF = 0;          				 // Clear UART interrupt flag
  }
}


//
// Send a command to the Bluetooth Module. The first argument is the command string
// to be sent to the Bluetooth module. The second argument is the Response expected
// from the module (can be CMD, AOK, END, or CONN)
//
void Send_Command(char *msg, unsigned char Resp)
{
   do
   {
       UART1_Write_Text(msg);
       UART1_Write(0x0D);
       Delay_Ms(500);
   }while(Response != Resp);
}



//
// Start of MAIN program
//
void main()
{
     ANSELB = 0;                      			// Configure PORTB as digital
     ANSELC = 0;                      			// Configure PORTC as digital
     TRISC1_bit = 0;                  			// Configure RC1 as an output
 //
 // Enable UART receive interrupts
 //
     PIE1.RC1IE = 1;                   			// Clear UART1 interrupt flag
     INTCON.PEIE = 1;                  			// Enable UART1 interrupts
     INTCON.GIE = 1;                  			// Enable global interrupts

     LCD_Init();                    			// Initialize LCD
     Lcd_Cmd(_LCD_CLEAR);           			// Clear LCD
     Lcd_Cmd(_LCD_CURSOR_OFF);      			// Cursor off
     Lcd_Out(1,1,"Cmd Mode");       			// Display message
     
     Uart1_Init(9600);              			// Initialzie UART to 9600 Baud
     
     Command_Mode = 1;              			// We are getting into Command Mode
     RESET = 0;                     			// Reset the Bluetooth module
     Delay_Ms(100);
     RESET = 1;
     Delay_Ms(1000);               			// End of Resetting the Bluetooth module

     do                            			// Send $$$ until in command mode
     {
        UART1_Write_Text("$$$");     			// Get into command mode
        Delay_Ms(1000);
     }while(Response != CMD);

     Send_Command("SN,Bluetooth-Testing",AOK);          // Set device name
     Send_Command("SO,Slave", AOK);                     // Set extended status
     Send_Command("SM,0", AOK);                         // Set into slave mode
     Send_Command("SA,1", AOK);                         // Enable authentication
     Send_Command("SP,1234", AOK);                      // Set pass code
     Send_Command("---", END);                          // Exit command mode
     Lcd_Out(1,1,"Connecting");                         // Display message
     while(Response != CONN);     			// Wait until connected
     
     Command_Mode = 0;            			 // Now we are in Data mode
     Data_Received = 0;
     
     Lcd_Out(1,1,"Connected ");

     for(;;)                                              // Display received msg on the LCD
     {
        i = 0;
        while(Data_Received == 0);          		  // Wait until data up to CR is received
        Data_Received = 0;                  		  // Clear data receivd flag
        Lcd_Cmd(_LCD_CLEAR);                		  // Clear LCD
        Lcd_Out(1,1,"Received Data:");                	  // Display "Received Data:" on row 1
        Lcd_Out(2,1,Txt);                             	  // Displayed received data
        for(i = 0; i < 15; i++)Txt[i] = 0;           	  // Clear buffer for next time
     }
}
