/******************************************************************************
                        	DC MOTOR CONTROL
                        	================

In this project a DC motor is connected to PORTB of a microcontroller via an H-bridge circuit,
constructed from 4 MOSFET transistors.

Two push-button switches are used to control the motor. Normally the motor rotates 
anticlockwise. Pressing the DIR button changes the direction of rotation. Pressing the ON/OFF
button stops the motor.

Author:   Dogan Ibrahim
Date:     October 2013
File:     MIKROC-DCMTR1.C
*******************************************************************************/
#define ONOFF PORTA.RA0
#define DIR PORTA.RA1
#define ON 1
#define ClockWise 0x05
#define AntiClockWise 0x0A
//
// Two buttons are used to control the motor movements: The motor turns clockwise,
// anticlockwise, or stops
//
void main()
{
     ANSELA = 0;                       			// Configure PORTA as digital
     ANSELB = 0;                       			// Configure PORTB as digital
     TRISB = 0;                       			// Configure PORTB as output
     TRISA = 3;                       			// Configure RA0 and RA1 as inputs

     for(;;)                        			// Do forever
     {
        if(ONOFF == ON)                 		// The motor is normally ON
        {
           if(DIR == 0)
             PORTB = ClockWise;
           else 
             PORTB = AntiClockWise;
        }
        else
          PORTB = 0;               			// If the ON/OFF button is pressed, stop the motor
     }
}
