/******************************************************************************
                    	OPTICAL ENCODER MOTOR SPEED MEASUREMENT
                    	=========================================

In this project a homemade optical encoder is uswed. The encoder has 8 holes and is connected
to the shaft of  DC motor. Infrared light source and detectors are used and connected to a
microcontroller to count the number of holes passing in-front of the detector. The speed is then calculated in RPM and is displayed on the LCD.

Timer 0 is configured in 16-bit mode to generate interrupts at every 100ms. The number of holes
passing in-front of the detector in 100ms is counted and then the RPM is calculated as described in the text.

Author:   Dogan Ibrahim
Date:     October 2013
File:     MIKROC-ENCODER.C
*******************************************************************************/
// LCD module connections
sbit LCD_RS at RB4_bit;
sbit LCD_EN at RB5_bit;
sbit LCD_D4 at RB0_bit;
sbit LCD_D5 at RB1_bit;
sbit LCD_D6 at RB2_bit;
sbit LCD_D7 at RB3_bit;

sbit LCD_RS_Direction at TRISB4_bit;
sbit LCD_EN_Direction at TRISB5_bit;
sbit LCD_D4_Direction at TRISB0_bit;
sbit LCD_D5_Direction at TRISB1_bit;
sbit LCD_D6_Direction at TRISB2_bit;
sbit LCD_D7_Direction at TRISB3_bit;
// End LCD module connections

#define ENCODER PORTA.RA0
unsigned int RPM, Cnt = 0;
unsigned char Txt[] = "RPM=       ";
unsigned char DsplyCnt = 0;

void interrupt(void)
{
     TMR0H = 0x3C;                  				// Reload TMR0 for 100ms interrupts
     TMR0L = 0xB0;
     RPM = 75*Cnt;                   				// Speed in RPM
     IntToStr(RPM, Txt+4);              			// Convert to string
     Cnt = 0;
     DsplyCnt++;
     if(DsplyCnt == 10)                  			// 1 second ?
     {
       DsplyCnt = 0;
       Lcd_Cmd(_LCD_CLEAR);					// Clear LCD
       Ltrim(Txt+4);						// Remove leading spaces
       Lcd_Out(1,1,Txt);            				// Display speed every second
     }
     INTCON.T0IF = 0;               				// Re-enable TMR0 interrupts
}

//
// Two buttons are used to control the motor movements: The motor turns clockwise,
// anticlockwise, or stops
//
void main()
{
     ANSELA = 0;                       				// Configure PORTA as digital
     ANSELB = 0;                       				// Configure PORTB as digital
     TRISB = 0;                       				// Configure PORTB as output
     TRISA = 1;                       				// Configure RA0 as input
     
     Lcd_Init();                      				// Initialize LCD
 //
 // Configure TMR0 is 16 bit mode to generate interrupts every 100ms
 //
     T0CON = 0x81;                   				// 16-bit mode, prescaler = 4
     TMR0H = 0x3C;                   				// Load 0x3CB0 for 100ms interrupts
     TMR0L = 0xB0;
     INTCON = 0xA0;                  				// Enable TMR0 and global interrupts 
     
     while(1)
     {
        while(ENCODER == 0);         				// Wait if 0
        Cnt++;                       				// Increment encoder count
        while(ENCODER == 1);         				// Wait if 1
     }
}
