/******************************************************************************
                	4-DIGIT 7-SEGMENT DISPLAY EVENT COUNTER
                	=======================================

In this project four common cathode 7-segment LED displays are connected to PORTD of a PIC18F45K22 microcontroller and the microcontroller is operated from an 8MHz crystal. 
Four PORTA pins are used to enable/disable the LEDs. In addition, external interrupt input
INT0 (RB0) is used to receive external events. An event is assumed to occur if pin INT0 goes
from logic 0 to logic 1.

The program uses two ISR routines: the timer routine is used to refresh the Display every 5ms. External interrupt ISR is used to increment the event count. The event count is displayed continuously on the 7-segment displays.


Author:           Dogan Ibrahim
Date:             August 2013
File:             MIKROC-EVENT1.C
*******************************************************************************/
#define DIGIT1 PORTA.RA0
#define DIGIT2 PORTA.RA1
#define DIGIT3 PORTA.RA2
#define DIGIT4 PORTA.RA3

unsigned int Cnt = 0;
unsigned char flag = 0;
unsigned int D1,D2,D3,D4,D5,D6;

//
// This function finds the bit pattern to be sent to the port to display a number
// on the 7-segment LED. The number is passed in the argument list of the function.
//
unsigned char Display(unsigned char no)
{
    unsigned char Pattern;
    unsigned char SEGMENT[] = {0x3F,0x06,0x5B,0x4F,0x66,0x6D,0x7D,0x07,0x7F,0x6F};

    Pattern = SEGMENT[no];                             		// Pattern to return
    return (Pattern);
}

//
// Interrupt Service Routine
//
void interrupt (void)
{
     if(INTCON.TMR0IF == 1)                                     // If Timer interrupt occured
     {
       TMR0L = 100;                                      	// Re-load timer register
       INTCON.TMR0IF = 0;                                       // Clear timer interrupt flag
       switch(flag)
       {
         case 0:
         {
           DIGIT1 = 0;                                  	// Disable digit 1
           D1 = Cnt / 1000;                         		// 1000s  digit
           PORTD = Display(D1);                     		// Send to PORTD
           DIGIT4 = 1;                              		// Enable digit 4
           flag = 1;
           break;
         }
         case 1:
         {
           DIGIT4 = 0;                               		// Disable digit 4
           D2 = Cnt % 1000;
           D3 = D2 / 100;                           		// 100s digit
           PORTD = Display(D3);                     		// Send to PORTD
           DIGIT3 = 1;                              		// Enable digit 3
           flag = 2;
           break;
         }
         case 2:
         {
           DIGIT3 = 0;                               		// Disable digit 3
           D4 = D2 % 100;
           D5 = D4/10;                              		// 10s digit
           PORTD = Display(D5);                     		// Send to PORTD
           DIGIT2 = 1;                              		// Enable digit 2
           flag = 3;
           break;
         }
         case 3:
         {
           DIGIT2 = 0;                               		// Disable digit 2
           D6 = D4 % 10;
           PORTD = Display(D6);                     		// Send to PORTD
           DIGIT1 = 1;                              		// Enable digit 1
           flag = 0;
           break;
         }
       }  
     }
     if(INTCON.INT0IF == 1)                             	// If external interrupt occured
     {
         Cnt++;                                         	// Increment event count
         INTCON.INT0IF = 0;                                   	// Clear ext interrupt flag
     }
}



//
// Start of MAIN Program
//
void main()
{

     ANSELA = 0;                                 	  	// Configure PORTA as digital
     ANSELD = 0;                                 	  	// Configure PORTD as digital
     ANSELB = 0;                                          	// Configure PORTB as digital
     TRISA = 0;                                  	  	// Configure PORTA as outputs
     TRISD = 0;                                  	  	// Configure PORTD as outputs
     TRISB = 1;                                          	// RB0 is event input

     DIGIT1 = 0;                                 	  	// Disable digit 1
     DIGIT2 = 0;                                 	  	// Disable digit 2
     DIGIT3 = 0;                                 	  	// Disable digit 3
     DIGIT4 = 0;                                 	  	// Disable digit 4
 //
 // Configure TIMER0 interrupts
 //
     T0CON = 0xC5;                                        	// TIMER0 in 8-bit mode
     TMR0L = 100;                                         	// Load Timer register
 //
 // Configure External interrupts and enable interrupts
 //
     INTCON = 0xB0;
     
     for(;;)                                     	  	// Endless loop
     {                                                    	// Wait and process interrupts
     }
 }
