/*******************************************************************************
                          	GPS WITH LCD OUTPUT
                          	===================

In this project a GPS is connected to a PIC18F45K22 microcontroller. The co-ordinates (latitude
and longitude) of the current location are read and displayed on an LCD.

The GPS Click Board (www.mikroe.com) is used in this project together with the EasyPIC V7 development board. The GPS board is connected to one of the mikroBUS connectors on the development board.

The LCD is connected to PORTB of the microcontroller as in the previous projects.

The communications between the GPS and the microcontroller is by using the serial port.
TX pin (RC6) and the RX pin (RC7) of the microcontroller are connected to the corresponding
 serial pins of the GPS.

The latitude and longitude are determined by decoding the NMEA sentence $GPGLL.  An example
is given below:

        $GPGLL,5127.35744,N,00003.13373,E,103109.00,A,A*65

        5127.35744,N             Latitude 51 deg. 27.35744 min. North
        00003.1276,E             Longitude 000 deg. 03.1276 min. East
        103109.00                Fix taken at 10:31:09 UTC
        A                        Data valid
        A                        Data autonomous
        *65                      Checksum
        
The pogram assumes fixed length NMEA sentence.

Author:   Dogan Ibrahim
Date:     September 2013
File:     MIKRO-GPS.C
*******************************************************************************/
// LCD module connections
sbit LCD_RS at RB4_bit;
sbit LCD_EN at RB5_bit;
sbit LCD_D4 at RB0_bit;
sbit LCD_D5 at RB1_bit;
sbit LCD_D6 at RB2_bit;
sbit LCD_D7 at RB3_bit;

sbit LCD_RS_Direction at TRISB4_bit;
sbit LCD_EN_Direction at TRISB5_bit;
sbit LCD_D4_Direction at TRISB0_bit;
sbit LCD_D5_Direction at TRISB1_bit;
sbit LCD_D6_Direction at TRISB2_bit;
sbit LCD_D7_Direction at TRISB3_bit;
// End LCD module connections

void main()
{
     unsigned char buffer[50];
     unsigned char i,flag,c;
     unsigned char Lat[13], Lon[13];
     unsigned char gps[]="$GPGLL,";
     
     ANSELB = 0;
     ANSELC = 0;

     LCD_Init();                                    		// Initialize LCD
     Lcd_Cmd(_LCD_CURSOR_OFF);                      		// Disable cursor
     Lcd_Out(1,1,"INVALID DATA");                   		// Display INVALID DATA to start with
     
     Uart1_Init(9600);                           		// Baud=9600

     for(;;)                                        		// Endless loop
     {  
        for(i = 0; i < 50; i++)buffer[i] = 0;			// Clear the buffer
        
        i=0; 
        flag=0;
//
// Read until "$GPGLL," is detected
//
        while(flag == 0)
        {
           if(Uart1_Data_Ready() == 1)
           {
             c = Uart1_Read();
             if(c == gps[i])
             {
               i++;
               if(i == 7)flag=1;
             }
             else i = 0;
           }
        }
//
// We come to this point when the string "$GPGLL," has been detected
//
        Uart1_Read_Text(buffer,"*",255);                        // Read until "*" detected
        if(buffer[37] == 'A')                                   // If the sentence is valid
        {                                                       // Get latitude Degrees
           Lat[0] = buffer[0];
           Lat[1] = buffer[1];
           Lat[2] = 178;                                        // Degree character
           Lat[3] = ' ';
           Lat[4] = ' ';
           for(i=0; i < 6; i++)Lat[5+i] = buffer[2+i];          // Get latitude minutes
           Lat[11] = buffer[11];                                // Get latitude direction
           Lat[12] = 0x0;                                       // Terminate the string
           
           Lon[0] = buffer[13];                                 // Get longitude Degrees
           Lon[1] = buffer[14];
           Lon[2] = buffer[15];
           Lon[3] = 178;                                        // Degree character
           Lon[4] = ' ';
           for(i=0; i < 6; i++)Lon[5+i] = buffer[16+i];         // Get longitude minutes
           Lon[11] = buffer[25];                                // Get longitude direction
           Lon[12] = 0x0;                                       // Terminate the string
           Lcd_Cmd(_LCD_CLEAR);                                 // Clear LCD
           Lcd_Out(1,1,"LAT=");                                 // Display LAT=
           Lcd_Out_Cp(Lat);                                     // Display the latitude
           Lcd_Out(2,1,"LON=");                                 // Display LON=
           Lcd_Out_Cp(Lon);                                     // Display the longitude
        }
        else                                                    // If invalid data
        {
          Lcd_Cmd(_LCD_CLEAR);                                  // Clear LCD
          Lcd_Out(1,1,"INVALID DATA");                          // Display INVALID DATA
        }
        Delay_Ms(2000);                                         // Wait 2 seconds
     }
}
