/******************************************************************************
                     	 ULTRASONIC HUMAN HEIGHT MEASUREMENT
                     	 ======================================

In this project the height of a person is found and displayed on a LCD. The project uses the
PING))) Ultrasonic distance measuring transducer pair, connected to RD0 pin of a PIC18F45K22 microcontroller, operated from an 8MHz crystal.

The LCD is connected to PORTB of the microcontroller.

The program assumes that the transducer pair is mounted on a pole 200cm above the ground
level where the person stands.

Author:      Dogan Ibrahim
Date:        September 2013
File:        MIKROC-HEIGHT.C
********************************************************************************/

// LCD module connections
sbit LCD_RS at LATB4_bit;
sbit LCD_EN at LATB5_bit;
sbit LCD_D4 at LATB0_bit;
sbit LCD_D5 at LATB1_bit;
sbit LCD_D6 at LATB2_bit;
sbit LCD_D7 at LATB3_bit;
sbit LCD_RS_Direction at TRISB4_bit;
sbit LCD_EN_Direction at TRISB5_bit;
sbit LCD_D4_Direction at TRISB0_bit;
sbit LCD_D5_Direction at TRISB1_bit;
sbit LCD_D6_Direction at TRISB2_bit;
sbit LCD_D7_Direction at TRISB3_bit;
// End LCD module connections

// Ultrasonic module connection
sbit Ultrasonic at RD0_bit;
sbit Ultrasonic_Direction at TRISD0_bit;
// End of Ultrasonic module connections

#define Pole_Height 200

void main()
{
     unsigned long Tm;
     unsigned char Tl, Th;
     unsigned int h, Person_Height;
     char Txt[7];

     ANSELB = 0;                               		// Configure PORTB as digital
     ANSELD = 0;                               		// Configure PORTD as digital

     Lcd_Init();                               		// Initialize LCD
     Lcd_Cmd(_LCD_CURSOR_OFF);                 		// Disable cursor
     Lcd_Out(1,1,"HEIGHT");                    		// Display heading
     Delay_Ms(2000);                           		// Wait 2 seconds
//
// Configure Timer 0 as a counter to operate in 16-bit mode with 1 microsecond
// count time. The prescaler is set to 2. The timer is stopped at this point.
//
     T0CON = 0x00;
//
// Start of program loop
// Send a pulse, start timer, get echo, stop timer, calculate distance and display
//
     for(;;)
     {
            Ultrasonic_Direction = 0;               	// RD0 in output mode
            TMR0H = 0;                              	// Clear high byte of timer
            TMR0L = 0;                              	// Clear low byte of timer

            Ultrasonic = 0;
            Delay_us(3);
            Ultrasonic = 1;                         	// Send a PULSE to Ultrasonic module
            Delay_us(5);
            Ultrasonic = 0;
            Ultrasonic_Direction = 1;               	// RD0 in input mode
            while(Ultrasonic == 0);                 	// Wait until echo is received
            T0CON.TMR0ON = 1;                       	// Start Timer0
            while(Ultrasonic == 1);
            T0CON.TMR0ON = 0;                       	// Stop Timer0
            Tl = TMR0L;                             	// Read timer low byte
            Th = TMR0H;                             	// Read timer high byte
            Tm = Th*256 + Tl;                       	// Timer as 16 bit value
            //
            // Now find the distance to person's head
            Tm = Tm / 2;                            	// Tm is half the time
            Tm = 34 * Tm;
            Tm = Tm / 1000;                         	// Divide by 1000
            h = (unsigned int)Tm;                   	// h is the distance to person's head
            Person_Height= Pole_Height - h;         	// Person's height
            //
            // Now display the height
            //
            IntToStr(Person_Height, Txt);          	// Convert into string to display
            Lcd_Cmd(_LCD_CLEAR);                       	// Clear LCD
            Lcd_Out(1,1, "Height (cm)");             	// Display heading
            Lcd_Out(2,1, Txt);                      	// Display the height
            Delay_Ms(1000);                         	// Wait 1 second
     }
}
