/******************************************************************************
                        High/Low Game Using Keypad and LCD
			==================================

This project implements the High/Low game using the 4 x 4 kaypad and an LCD.
The program generates a random number between 1 and 32767 and expects the
player to guess the number. The LCD displays "Guess Now.." on top row of the
display.

The player then guesses the number by entering a number via the keypad and then
pressing the ENTER key. If the guessed number is bigger than the generated number
the message "HIGH - Try Again" will be displayed on bottom row of the LCD.

If the guessed number is lower than the generated number then the message
"LOW - Try Again" will be generated on the bottom row of the LCD. If on the
other hand the player guesses the number correctly, the bottom row of the LCD
will display the message "Well Done..".

The game will re-start after 5 seconds delay.

The microcontroller in this project is PIC18F45K22 and is operated from an 8MHz
crystal as before. An 4x4 keypad is connected to PORTC. The LCD is connected to
PORTB.

Programmer:	Dogan Ibrahim
Date:           September 2013
File:           MIKROC-HILO.C

***************************************************************************/
// Declare LCD connections
sbit LCD_RS at RB4_bit;
sbit LCD_EN at RB5_bit;
sbit LCD_D4 at RB0_bit;
sbit LCD_D5 at RB1_bit;
sbit LCD_D6 at RB2_bit;
sbit LCD_D7 at RB3_bit;

sbit LCD_RS_Direction at TRISB4_bit;
sbit LCD_EN_Direction at TRISB5_bit;
sbit LCD_D4_Direction at TRISB0_bit;
sbit LCD_D5_Direction at TRISB1_bit;
sbit LCD_D6_Direction at TRISB2_bit;
sbit LCD_D7_Direction at TRISB3_bit;
// End of LCD connections

// Declare keypad connection
char keypadPORT at PORTC;
// End of keypad connection

#define ENTER_KEY 12

unsigned char kp, new_game;
unsigned int GuessNumber, PlayerNumber;
int Diff;


void main()
{
        unsigned char Txt1[4];

	ANSELB = 0;				             	// Configure PORT B as digital
	ANSELC = 0;                                          	// Configure PORTC as digital
	Keypad_Init();				             	// Initialize keypad library
	Lcd_Init();				             	// Initialize LCD
	Lcd_Out(1, 1, "High/Low Game");		             	// Display heading
	Delay_ms(2000);				             	// Wait 2 seconds

	new_game = 1;
	srand(5);				             	// Random number seed

        for(;;)					     		// DO FOREVER
       	{
 		if(new_game == 1)
  	        {
			Lcd_Cmd(_LCD_CLEAR);		   	// Clear LCD
			Lcd_Cmd(_LCD_CURSOR_OFF);		// Turn OFF cursor
			Lcd_Out(1, 1, "Guess Now..");		// Display "Guess Now.."
			GuessNumber = rand();			// Generate a random number
               	 }
		kp = 0;
		PlayerNumber = 0;
		Lcd_Out(2, 1, "");				// Position cursor at 1,1
		while(kp != ENTER_KEY)				// Until ENTER pressed
		{
                   do
                   {
                      kp = Keypad_Key_Click();			// Look for key press
                   }while(!kp);

                   if(kp != ENTER_KEY)				// If not ENTER key
                   {
               	  	if(kp > 4 && kp <9)kp = kp=kp-1;	// 5 is 4, 6 is 5.
                        if(kp > 8 && kp < 12)kp = kp-2;		// 7 is 9, 8 is 10
                        if(kp == 14)kp=0;			// 0 is14
                        PlayerNumber = 10*PlayerNumber + kp;
                        ByteToStr(kp, Txt1);
                        Txt1[0] = Txt1[2];			// Get the number
                        Txt1[1] = '\0';				// Make a string
                        Lcd_Out_Cp(Txt1);			// Display on LCD
                   }
                }

                Delay_ms(1000);					// Wait one second
                Diff = PlayerNumber - GuessNumber;		// Find the diff

                if(Diff > 0)					// Greater ?
                {
			Lcd_Out(2, 1, "HIGH - Try Again");
			new_game = 0;				// Not a new game
			Delay_ms(1000);
			Lcd_Out(2,1,"                ");	// Clear second row
                }

                if(Diff < 0)					// Less ?
                {
			Lcd_Out(2, 1, "LOW - Try Again");
			new_game = 0;				// Not a new game
			Delay_ms(1000);
			Lcd_Out(2,1,"                ");	// Clear second row
                }

                if(Diff == 0)					// Equal ?
                {
			Lcd_Out(2,1, "Well Done..");
			new_game = 1;				// New game
			Delay_ms(5000);				// Wait 5 seconds to
	        }						// re-start game
       }  							// End of for
     }								// End of program

