/*******************************************************************************
                  		CALCULATOR WITH KEYPAD AND LCD
                  		==============================

In this project a 4 x 4 keypad is connected to PORTC of a PIC18F45K22 microcontroller. Also an
LCD is connected to PORTB. The project is a simple calculator which can perform integer arithmetic.

The keys are organised as follows:

    0    1    2   3
    4    5    6   7
    8    9   10  11
    12  13  14   15

The keys are labelled as follows:

    1   2   3   4
    5   6   7   8
    9   0       Enter
    +   -   *   /


Author:   Dogan Ibrahim
Date:     August 2013
File:     MIKROC-KEYPAD.C
**************************************************************/
// LCD module connections
sbit LCD_RS at RB4_bit;
sbit LCD_EN at RB5_bit;
sbit LCD_D4 at RB0_bit;
sbit LCD_D5 at RB1_bit;
sbit LCD_D6 at RB2_bit;
sbit LCD_D7 at RB3_bit;

sbit LCD_RS_Direction at TRISB4_bit;
sbit LCD_EN_Direction at TRISB5_bit;
sbit LCD_D4_Direction at TRISB0_bit;
sbit LCD_D5_Direction at TRISB1_bit;
sbit LCD_D6_Direction at TRISB2_bit;
sbit LCD_D7_Direction at TRISB3_bit;
// End LCD module connections

#define MASK 0xF0
#define Enter 11
#define Plus 12
#define Minus 13
#define Multiply 14
#define Divide 15

//
// This function gets a key from the keypad and returns it to calling program
//
unsigned char getkeypad()
{
        unsigned char i, Key = 0;

        PORTC = 0x01;                        			// Start with column 1
        while((PORTC & MASK) == 0)           	        	// While no key pressed
        {
             PORTC = (PORTC << 1);           			// next column
             Key++;                          			// column number
             if(Key == 4)
             {
                 PORTC = 0x01;               			// Back to column 1
                 Key = 0;
             }
        }
        Delay_Ms(20);                        			// Switch debounce

        for(i = 0x10; i !=0; i <<=1)         	        	// Find the key pressed
        {
           if((PORTC & i) != 0)break;
           Key = Key + 4;
        }

        PORTC = 0x0F;
        while((PORTC & MASK) != 0);             		// Wait until key released
        Delay_Ms(20);                        			// Switch debounce

        return (Key);                          			// Return key number
}



//
// Start of MAIN program
//
void main()
{
     unsigned char MyKey, i,j,op[12];
     unsigned long Calc, Op1, Op2;
     char *lcd;
     
     ANSELB = 0;                                        	// Configure PORTB as digital
     ANSELC = 0;                                        	// Configure PORTC as digital
     TRISB = 0;                              			// PORTB are outputs (LCD)
     TRISC = 0xF0;                           			// RC4-RC7 are inputs

//
// Configure LCD
//
       Lcd_Init();                     		      		// Initialize LCD
       Lcd_Cmd(_LCD_CLEAR);					// Clear LCD
       Lcd_Out(1,1,"CALCULATOR");            	       		// Display CALCULATOR
       Delay_ms(2000);                       			// Wait 2 seconds
       Lcd_Cmd(_LCD_CLEAR);                   			// Clear display
//
// Program loop
//
     for(;;)                                 			// Endless loop
     {
        MyKey = 0;
        Op1 = 0;
        Op2 = 0;

        Lcd_Out(1,1,"No1: ");                			// Display No1:
        while(1)                             			// Get first no
        {
           MyKey = getkeypad();
           if(MyKey == Enter)break;          			// If ENTER pressed
           MyKey++;                                     	// Key number pressed
           if(MyKey == 10)MyKey = 0;				// If 0 key pressed
           Lcd_Chr_Cp(MyKey + '0');
           Op1 = 10*Op1 + MyKey;             			// First number in Op1
        }

        Lcd_Out(2,1,"No2: ");                			// Display No2:
        while(1)                             			// Get second no
        {
           MyKey = getkeypad();
           if(MyKey == Enter)break;          			// If ENTER pressed
           MyKey++;
           if(MyKey == 10)MyKey = 0;				// If 0 key pressed
           Lcd_Chr_Cp(MyKey + '0');
           Op2 = 10*Op2 + MyKey;             			// Second number in Op2
        }

        Lcd_Cmd(_LCD_CLEAR);                  			// Clear LCD
        Lcd_Out(1,1,"Op: ");                 			// Display Op:

        MyKey = getkeypad();                 			// Get operation
        Lcd_Cmd(_LCD_CLEAR);
        Lcd_Out(1,1,"Res=");                 			// Display Res=
        switch(MyKey)                        			// Perform the operation
        {
          case Plus:
                  Calc = Op1 + Op2;          			// If ADD
                  break;
          case Minus:
                  Calc = Op1 - Op2;          			// If Subtract
                  break;
          case Multiply:
                  Calc = Op1 * Op2;          			// If Multiply
                  break;
          case Divide:
                  Calc = Op1 / Op2;          			// If Divide
                  break;
        }

        LongToStr(Calc, op);                 			// Convert to string in op
        lcd = Ltrim(op);                                 	// Remove leading blanks

        Lcd_Out_Cp(lcd);                     			// Display result
        Delay_ms(5000);                      			// Wait 5 seconds
        Lcd_Cmd(_LCD_CLEAR);                           		// Clear LCD
     }
}

