/*******************************************************************************
                  		CALCULATOR WITH KEYPAD AND LCD
                  		===============================

In this project a 4 x 4 keypad is connected to PORTC of a PIC18F45K22 microcontroller. Also an
LCD is connected to PORTB. The project is a simple calculator which can perform integer
arithmetic.

The keys are labelled as follows:

    1   2   3   4
    5   6   7   8
    9   0       Enter
    +   -   *   /

In this version of the program built-in keypad libray is used

Author:   Dogan Ibrahim
Date:     August 2013
File:     MIKROC-KEYPAD2.C
*******************************************************************************/
// LCD module connections
sbit LCD_RS at RB4_bit;
sbit LCD_EN at RB5_bit;
sbit LCD_D4 at RB0_bit;
sbit LCD_D5 at RB1_bit;
sbit LCD_D6 at RB2_bit;
sbit LCD_D7 at RB3_bit;

sbit LCD_RS_Direction at TRISB4_bit;
sbit LCD_EN_Direction at TRISB5_bit;
sbit LCD_D4_Direction at TRISB0_bit;
sbit LCD_D5_Direction at TRISB1_bit;
sbit LCD_D6_Direction at TRISB2_bit;
sbit LCD_D7_Direction at TRISB3_bit;
// End LCD module connections

// Keypad module connections
char keypadPort at PORTC;
// End of keypad module connections

#define Enter 12
#define Plus 13
#define Minus 14
#define Multiply 15
#define Divide 16

//
// Start of MAIN program
//
void main()
{
     unsigned char MyKey, i,j,op[12];
     unsigned long Calc, Op1, Op2;
     char *lcd;
     
     ANSELB = 0;                                        	// Configure PORTB as digital
     ANSELC = 0;                                        	// Configure PORTC as digital
     TRISB = 0;                              			// PORTB are outputs (LCD)
     TRISC = 0xF0;                           			// RC4-RC7 are inputs

     Keypad_Init();                                      	// Initialize keypad library
//
// Configure LCD
//
       Lcd_Init();                     		       		// Initialize LCD
       Lcd_Cmd(_LCD_CLEAR);
       Lcd_Out(1,1,"CALCULATOR");            	       		// Display CALCULATOR
       Delay_ms(2000);                       			// Wait 2 seconds
       Lcd_Cmd(_LCD_CLEAR);                   			// Clear display
//
// Program loop
//
     for(;;)                                 			// Endless loop
     {
        MyKey = 0;
        Op1 = 0;
        Op2 = 0;

        Lcd_Out(1,1,"No1: ");                			// Display No1:
        while(1)                             			// Get first no
        {
          do
               MyKey = Keypad_Key_Click();
          while(!MyKey);
           if(MyKey == Enter)break;          			// If ENTER pressed
           if(MyKey == 10)MyKey = 0;				// If 0 key pressed
           Lcd_Chr_Cp(MyKey + '0');
           Op1 = 10*Op1 + MyKey;             			// First number in Op1
        }

        Lcd_Out(2,1,"No2: ");                			// Display No2:
        while(1)                             			// Get second no
        {
          do
               MyKey = Keypad_Key_Click();
          while(!MyKey);
           if(MyKey == Enter)break;          			// If ENTER pressed
           if(MyKey == 10)MyKey = 0;				// If 0 key pressed
           Lcd_Chr_Cp(MyKey + '0');
           Op2 = 10*Op2 + MyKey;             			// Second number in Op2
        }

        Lcd_Cmd(_LCD_CLEAR);                  			// Clear LCD
        Lcd_Out(1,1,"Op: ");                 			// Display Op:

        do
              MyKey = Keypad_Key_Click();                 	// Get operation
        while(!MyKey);
        Lcd_Cmd(_LCD_CLEAR);
        Lcd_Out(1,1,"Res=");                 			// Display Res=
        switch(MyKey)                        			// Perform the operation
        {
          case Plus:
                  Calc = Op1 + Op2;          			// If ADD
                  break;
          case Minus:
                  Calc = Op1 - Op2;          			// If Subtract
                  break;
          case Multiply:
                  Calc = Op1 * Op2;          			// If Multiply
                  break;
          case Divide:
                  Calc = Op1 / Op2;          			// If Divide
                  break;
        }

        LongToStr(Calc, op);                 			// Convert to string in op
        lcd = Ltrim(op);                                 	// Remove leading blanks

        Lcd_Out_Cp(lcd);                     			// Display result
        Delay_ms(5000);                      			// Wait 5 seconds
        Lcd_Cmd(_LCD_CLEAR);                           		// Clear LCD
     }
}
