/*******************************************************************************
                  		VOLTMETER WITH LCD DISPLAY
                  		==========================

In this project an LCD is connected to PORTB. Also, input port AN0 is used as analog input.
Voltage to be measured is applied to AN0. The microcontroller reads the analog voltage,
converts into digital, and then displays on the LCD.

Analog input range is 0 to 5V. A PIC18F45K22 type microcontroller is used in this project,
operated with an 8MHz crystal.

Analog data is read using the Adc_Read built-in function.

The LCD is connected to the microcontroller as follows:

Microcontroller      LCD
===============      ===

    RB0              D4
    RB1              D5
    RB2              D6
    RB3              D7
    RB4              R/S
    RB5              E


Author:   	Dogan Ibrahim
Date:     	August 2013
File:     	MIKROC-LCD1.C
**************************************************************/
// LCD module connections
sbit LCD_RS at RB4_bit;
sbit LCD_EN at RB5_bit;
sbit LCD_D4 at RB0_bit;
sbit LCD_D5 at RB1_bit;
sbit LCD_D6 at RB2_bit;
sbit LCD_D7 at RB3_bit;

sbit LCD_RS_Direction at TRISB4_bit;
sbit LCD_EN_Direction at TRISB5_bit;
sbit LCD_D4_Direction at TRISB0_bit;
sbit LCD_D5_Direction at TRISB1_bit;
sbit LCD_D6_Direction at TRISB2_bit;
sbit LCD_D7_Direction at TRISB3_bit;
// End LCD module connections


void main()
{
     unsigned long Vin, mV;
     unsigned char op[12];
     unsigned char i,j,lcd[5];
     char *str;
     
     ANSELB = 0;                                   	// Configure PORTB as digital
     ANSELA = 1;                                   	// Configure RA0 as analog
     TRISB = 0;                       		   	// Configure PORTB as outputs
     TRISA = 1;                    		   	// Configure RA0 as input

//
// Configure LCD
//
     Lcd_Init();                		   	// LCD is connected to PORTB
     Lcd_Cmd(_LCD_CLEAR);
     Lcd_cmd(_LCD_CURSOR_OFF);                    	// Hide cursor
     Lcd_Out(1,1,"VOLTMETER");
     Delay_ms(2000);
//
// Configure A/D converter. AN0 is used in this project
//
     ADCON1 = 0x80;                   		  	// Use AN0 and Vref=+5V
//
// Program loop
//
     for(;;)                          		  	// Endless loop
     {
        Lcd_Cmd(_LCD_CLEAR);                      	// Clear LCD
        Vin = Adc_Read(0);            		  	// Read from channel 0 (AN0)
        Lcd_Out(1,1,"mV = ");         		  	// Display "mV = "
        mV = (Vin * 5000) >> 10;      			// mv = Vin x 5000 / 1024
        LongToStr(mV,op);             		  	// Convert to string in "op"
        str = Ltrim(op);                          	// Remove leading spaces
//
// Display result on LCD
//
        Lcd_Out(1,6,str);			  	// Output to LCD
        Delay_ms(1000);               		  	// Wait 1 second
     }
}
