/*******************************************************************************
                           	CREATING CUSTOM FONT ON LCD
                           	===========================

This project displays a custom font on the LCD. A "Right arrow" is displayed with text as shown below:

   		Right arrow  <up arrow symbol>

The font has been created using the mikro C font editor.

In this project a HD44780 controller based LCD is connected to a PIC18F45K22 type microcontroller, operated from an 8MHz crystal.

The LCD is connected to PORT B of the microcontroller as follows:

  LCD pin   Microcontroller pin
    D4           RB0
    D5           RB1
    D6           RB2
    D7           RB3
    R/S          RB4
    E            RB5

R/W pin of the LCD is not used and is connected to GND. The brightness of the LCD is
controlled by connecting the arm of a 10K potentiometer to pin Vo of the LCD. Other
pins of the potentiometer are connected to power and ground.


Author:  Dogan Ibrahim
Date:    August 2013
File:    MIKROC-LCD2.C
****************************************************************************/
// Start of LCD module connections
sbit LCD_RS at RB4_bit;
sbit LCD_EN at RB5_bit;
sbit LCD_D4 at RB0_bit;
sbit LCD_D5 at RB1_bit;
sbit LCD_D6 at RB2_bit;
sbit LCD_D7 at RB3_bit;

sbit LCD_RS_Direction at TRISB4_bit;
sbit LCD_EN_Direction at TRISB5_bit;
sbit LCD_D4_Direction at TRISB0_bit;
sbit LCD_D5_Direction at TRISB1_bit;
sbit LCD_D6_Direction at TRISB2_bit;
sbit LCD_D7_Direction at TRISB3_bit;
// End of LCD module connections

//
// The following code is generated automatically by the mikroC compiler font editor
//
 const char character[] = {0,4,2,31,2,4,0,0};

void CustomChar(char pos_row, char pos_char) {
  char i;
    Lcd_Cmd(64);
    for (i = 0; i<=7; i++) Lcd_Chr_CP(character[i]);
    Lcd_Cmd(_LCD_RETURN_HOME);
    Lcd_Chr(pos_row, pos_char, 0);
}

//
// Start of Main program
//
void main()
{

     ANSELB = 0;                        			// Configure PORT B as digital
     TRISB = 0;                         			// Configure PORT B pins as output

     Lcd_Init();                        			// Initialize LCD
     Lcd_Cmd(_LCD_CLEAR);               			// Clear display
     Lcd_Cmd(_LCD_CURSOR_OFF);          			// Cursor off

     Lcd_Out(1, 1, "Right arrow  ");      			// Display text "Right arrow  "
     CustomChar(1, 13);                 			// Display the "right arrow" symbol

     while(1);                          			// End of program, wait here forever
}
