/******************************************************************************
                  		VOLTMETER WITH LED DISPLAYS
                  		===========================

In this project 5 LEDs are connected to PORTD. Also, input port RA0 (AN0) is used as analog input. Voltage to be measured is applied to this pin. The microcontroller reads the analog voltage, 
converts into digital, and then turns ON one of the LEDs to indicate the voltage range.

Analog input range is 0 to 5V. A PIC18F45K22 type microcontroller is used in this project,
operated with an 8MHz cystal.

Analog data is read using the mikroC Pro for PIC built-in function Adc_Read.


Author:   	Dogan Ibrahim
Date:     	August 2013
File:     	MIKROC-LED11.C
*******************************************************************************/
#define LED01V PORTD.RD0
#define LED12V PORTD.RD1
#define LED23V PORTD.RD2
#define LED34V PORTD.RD3
#define LED45V PORTD.RD4

void main()
{
     unsigned long Vin, mV;

     ANSELD = 0;                                   	// Configure PORTD as digital
     ANSELA = 1;                                   	// COnfigure RA0 as analog
     TRISD = 0;                       		   	// Configure PORTD as outputs
     TRISA = 1;                    		   	// Configure RA0 as input

//
// Configure A/D converter. Channel 0 (RA0, or AN0) is used in this project
//
     ADCON2 = 0x80;                   		   	// Right justify the result
     
     for(;;)                          		   	// Endless loop
     {
        PORTD = 0;                                 	// Clear LEDs to start with
        Vin = Adc_Read(0);            		   	// Read from channel 0 (AN0)
        mV = (Vin * 5000) >> 10;      			// mv = Vin x 5000 / 1024
//
// Find which LED to turn ON
//
        if(mV >= 0 && mV <= 1000)                   	// Between 1 - 2V
        {
           LED01V = 1;                             	// Turn ON LED0-1V
        }
        else if(mV > 1000 && mV <= 2000)           	// Between 2 - 3V
        {
           LED12V = 1;                             	// Turn ON LED1-2V
        }
        else if(mV > 2000 && mV <= 3000)           	// Between 2 - 3V
        {
          LED23V = 1;                              	// Turn ON LED2 -3V
        }
        else if(mv > 3000 && mV <= 4000)           	// Between 3 - 4V
        {
          LED34V = 1;                              	// Turn ON LED3-4V
        }
        else if(mV > 4000 && mV <= 5000)           	// Between 4 - 5V
        {
          LED45V = 1;                              	// Turn ON LED4-5V
        }
        Delay_Ms(10);                              	// 10ms delay
     }
}
