/*******************************************************************************
                             		LOGIC PROBE
                             		===========

This is a logic probe project. In this project 2 coloured LEDs are connected to PORTC pins RC6
(RED) and RC7 (GREEN). A transistor is used at the front end of the project. Pin RC4 is
connected to the base of the transistor. The emitter of the transistor is connected to pin RC3
of the microcontroller. Table 7.1 in the text explains how the logic state of the probe signal is
detected.

If the logic probe is at logic 0 then the RED LED is turned ON. If the logic level is 1 the GREEN
LED is turned ON. If the probe is not connected to any logic signal or if the signal is at 
high-impedance state then none of the LEDs turn on.


Author:      Dogan Ibrahim
Date:        August 2013
File:        MIKROC-LED4-1.C
*******************************************************************************/
#define RED_LED PORTC.RC6
#define GREEN_LED PORTC.RC7
#define TSTOUT PORTC.RC4
#define TSTIN PORTC.RC3

void main()
{
     ANSELC = 0;                         			// Configure PORTC as digital
     TRISC6_bit = 0;                     			// Configure RC6 as output
     TRISC7_bit = 0;                     			// Configure RC7 as output
     TRISC4_bit = 0;                     			// Configure RC4 as output
     TRISC3_bit = 1;                     			// Configure RC3 as input

     for(;;)                             			// Endless loop
     {
        TSTOUT = 1;                      			// Set RC4 = 1
        Delay_ms(1);                     			// Small delay
        if(TSTIN == 1)                  			 // Check RC3. If RC3 = 1
        {
          TSTOUT = 0;                    			// Set RC4 = 0
          Delay_ms(1);                   			// Small delay
          if(TSTIN == 0)                 			// Check RC3. If RC3 = 0
          {
            GREEN_LED = 0;               			// High-impedance state
            RED_LED = 0;                 			// High-impedance state
          }
          else
          {
            RED_LED = 0;
            GREEN_LED = 1;               			// Probe at logic 1
          }
        }
        else
        {
          TSTOUT = 0;
          Delay_ms(1);                   			// Small delay
          if(TSTIN == 0)
          {
            GREEN_LED = 0;
            RED_LED = 1;                 			// Probe at logic 0
          }
        }
     }
}
