/*******************************************************************************
                              		TWO DICE
                              		========

In this project 7 LEDs are connected to PORT C of a PIC18F452 microcontroller and 7 LEDs to
PORTD. The microcontroller is operated from an 8MHz crystal. The LEDs are organised as the
faces of a real dice. When a push-button switch connected to RB0 is pressed a dice pattern is displayed on the LEDs. The display remains in this state for 3 seconds and after this period the
LEDs all turn OFF to indicate that the system is ready for the button to be pressed again.

In this program a pseudo random number generator function is used to generate the dice
numbers between 1 and 6.

Author:   	Dogan Ibrahim
Date:     	August 2013
File:     	MIKROC-LED6.C
*******************************************************************************/

#define Switch PORTB.F0
#define Pressed 0

//
// This function generates a pseudo random integer number between 1 and Lim
//
unsigned char Number(int Lim, int Y)
{
    unsigned char Result;
    static unsigned int Y;

    Y = (Y * 32719 + 3) % 32749;
    Result = ((Y % Lim) + 1);
    return Result;
}


//
// Start of MAIN program
//
void main()
{
     unsigned char J,Pattern,Seed = 1;
     unsigned char DICE[] = {0,0x08,0x22,0x2A,0x55,0x5D,0x77};

     ANSELC = 0;                                	// Configure PORTC as digital
     ANSELD = 0;                                	// Configure PORTD as digital
     ANSELB = 0;                                	// Configure PORTB as digital
     TRISC = 0;                       			// Configure PORT as outputs
     TRISD = 0;                       			// Configure PORTD as outputs
     TRISB = 1;                       			// Configure RB0 as input
     PORTC = 0;                       			// Turn OFF all LEDs
     PORTD = 0;                       			// Turn OFF all LEDs

     for(;;)                          			// Endless loop
     {
          if(Switch == Pressed)       			// Is switch pressed ?
          {
            J = Number(6,seed);       			// Generate first dice number
            Pattern = DICE[J];        			// Get LED pattern
            PORTC = Pattern;          			// Turn on LEDs for first dice
            J = Number(6,seed);       			// Generate second dice number
            Pattern = DICE[J];        			// Get LED pattern
            PORTD = Pattern;          			// Turn on LEDs for second dice
            Delay_ms(3000);           			// Delay 3 seconds
            PORTC = 0;                			// Turn OFF all LEDs
            PORTD = 0;                			// Turn OFF all LEDS
           }
     }
}
