/**********************************************************************
                  		TWO DICE - FEWER I/O COUNT
                  		==========================

In this project LEDs are connected to PORT C of a PIC18F45K22 microcontroller and
the microcontroller is operated from an 8MHz crystal. The LEDs are organised as the
faces of a real dice. When a push-button switch connected to RB0 is pressed a dice
pattern is displayed on the LEDs. The display remains in this state for 3 seconds and
after this period the LEDs all turn OFF to indicate that the system is ready for the
button to be pressed again.

In this program a pseudo random number generator function is used to generate
the dice numbers between 1 and 6.

Author: 	Dogan Ibrahim
Date:     	August 2013
File:     	MIKROC-LED7.C
**********************************************************************/

#define Switch PORTB.RB0
#define Pressed 0

//
// This function generates a pseudo random integer Number between 1 and Lim
//
unsigned char Number(int Lim, int Y)
{
    	unsigned char Result;
    	static unsigned int Y;

    	Y = (Y * 32719 + 3) % 32749;
    	Result = ((Y % Lim) + 1);
    	return Result;
}


//
// Start of MAIN program
//
void main()
{
     unsigned char J,L,U,R,Seed = 1;
     unsigned char DICE[] = {0,0x08,0x01,0x09,0x06,0x0E,0x07};

     ANSELC = 0;                                		// Configure PORTC as digital
     ANSELB = 0;                                		// Configure PORTB as digital
     TRISC = 0;		            	        		// Configure PORTC as outputs
     TRISB = 1;            		        		// Configure RB0 as input
     PORTC = 0;                       				// Turn OFF all LEDs

     for(;;)                          				// Endless loop
     {
          if(Switch == Pressed) 	        		// Is switch pressed ?
          {
            	J = Number(6,seed);       			// Generate first dice number
            	L = DICE[J];              			// Get LED pattern
            	J = Number(6,seed);       			// Generate second dice number
            	U = DICE[J];              			// Get LED pattern
            	R = 16*U + L;             			// Bit pattern to send to PORT C
       	 	PORTC = R;                      		// Turn on LEDs for both dice
            	Delay_ms(3000);           			// Delay 3 seconds
            	PORTC = 0;                			// Turn OFF all LEDs
           }
     }
}
