/******************************************************************************
                    		CLOSED-LOOP ON/OFF MOTOR SPEED CONTROL
                    		======================================

In this project a DC motor is controlled using feedback and ON/OFF type simple control. The
motor speedis measured using a homemade optical encoder.

If the motor speed is higher than the desired speed than the motor supply is cut. If on the other
hand the motor speed islower than the desired speed than full power is applied to the motor.

The motor is connected to the microcontroller through a VN66AFD type MOSFET transistor. The
PWM module of the microcontroller is used to provide control pulses to the motor. An LCD is
connected to PORTB and the LCD shows the actual motor speed every second.

The optical encoider is connecte dto RA0 input of the microcontroller. The motor is driven from the
CCP1 (RC2) output.

Homemade optical encoder is used. The encoder has 8 holes and is connected to the shaft of
DC motor. Infrared light source and detectors are used and connected to a microcontroller to
count the number of holes passing in-front of the detector. The speed is then calculated in RPM
and is displayed on the LCD.

Author:   Dogan Ibrahim
Date:     October 2013
File:     MIKROC-MTRONOFF.C
*******************************************************************************/// LCD module connections
sbit LCD_RS at RB4_bit;
sbit LCD_EN at RB5_bit;
sbit LCD_D4 at RB0_bit;
sbit LCD_D5 at RB1_bit;
sbit LCD_D6 at RB2_bit;
sbit LCD_D7 at RB3_bit;

sbit LCD_RS_Direction at TRISB4_bit;
sbit LCD_EN_Direction at TRISB5_bit;
sbit LCD_D4_Direction at TRISB0_bit;
sbit LCD_D5_Direction at TRISB1_bit;
sbit LCD_D6_Direction at TRISB2_bit;
sbit LCD_D7_Direction at TRISB3_bit;
// End LCD module connections

#define ENCODER PORTA.RA0
unsigned int Cnt = 0;
unsigned int RPM, DesiredRPM, DeadBand;
unsigned char Txt[] = "RPM=       ";
unsigned char DsplyCnt = 0;

//
// This function sets the motor speed
//
void SetSpeed (unsigned char N)
{
   unsigned char Temp;

   CCPR1L = N >> 2;
   Temp = N & 0x03;
   Temp = Temp << 4;
   CCP1CON = Temp | 0x0C;
}

//
// This is the Timer0 interrupt service routine. Th eprogram jumps here every 10ms.
// The actual motor speed is compared with the desired speed and either the motor is
// operated with full voltage (maximum speed), or with low voltage (minimum speed). The
// motor speed is displayed on the LCD.
//
void interrupt(void)
{
     TMR0L = 100;                       			// Reload Timer register
     RPM = 750*Cnt;                   				// Actual motor speed (in RPM)
     Cnt = 0;
     /* IF SLOW */
     if(RPM < (DesiredRPM - Deadband))
     {
        SetSpeed(200);                   			// Rotate motor FAST
     }

     /* IF FAST */
     if(RPM > (DesiredRPM + Deadband))
     {
        SetSpeed(2);                   				// Rotate motor SLOW
     }
     DsplyCnt++;
     if(DsplyCnt == 100)                  			// 1 second (10msx100=1s) ?
     {
       DsplyCnt = 0;
       Lcd_Cmd(_LCD_CLEAR);                			// Clear LCD
       IntToStr(RPM,Txt+4);                  			// Convert into string
       Ltrim(Txt+4);                        			// Remove leading spaces
       Lcd_Out(1,1,Txt);                    			// Display speed every second
     }

     INTCON.T0IF = 0;               				// Re-enable TMR0 interrupts
}

//
// The desired RPM is set to 2000 and the DeadBand is set to 100 RPM
//
void main()
{
     ANSELA = 0;                       				// Configure PORTA as digital
     ANSELB = 0;                       				// Configure PORTB as digital
     ANSELC = 0;                      				// Configure PORTC as digital
     TRISA = 1;                        				// Configure RA0 as input
     TRISB = 0;                       				// Configure RB0 as input

     LCD_Init();                      				// Initialize LCD
//
// Configure the PWM module
//
     T2CON = 0x05;                   				// Set Timer 2 ON and prescaler to 4
     PR2 = 0x31;                     				// Load Timer 2 PR2 register with 49
     CCPR1L = 0;                     				// Motor OFF to start with
     TRISC = 0;                 				// Configure CCP1 (RC2) as output
     CCP1CON = 0x0C;                 				// Enable PWM module
     CCPTMRS0 = 0;                   				// Use Timer 2 for the PWM module (CCP1)
 //
 // Configure TMR0 in 8 bit mode to generate interrupts every 10ms
 //
     T0CON = 0xC6;                   				// 8-bit mode, prescaler = 128
     TMR0L = 100;                    				// Timer value for 10ms timer overflow
     INTCON = 0xA0;                  				// Enable TMR0 interrupts

     DesiredRPM = 2000;              				// Desired RPM
     DeadBand = 100;                 				// Deadband range

     while(1)
     {
        while(ENCODER == 0);         				// Wait if 0
        Cnt++;                       				// Increment encoder count
        while(ENCODER == 1);        				// Wait if 1
     }
}
