/*******************************************************************************
                                ON-OFF EMPERATURE CONTROL
                  		=========================

In this project the temperature of a plant is controlled using simple ON-OFF type controller.

The plant temperature is measured using an LM35DZ type analog temperature sensor. An
LCD helps to enter the set point temperature, and also displays the set point as well as the
actual plant temperature in real-time.

 A 4x4 keypad is used to set the desired temperature. If the set temperature is below the
actual plant temperature then the heater relay is turned ON. If on the other hand the set
temperature is above the actual plant temperature then the relay is turned OFF. A buzzer
sounds if the absolute difference between the desired and the actual temperatures is more
than 2 Degrees Centigrade.

The LCD is connected to PORTB
The buzzer is connected to pin RE1 through a transistor switch
The keypad is connected to PORTC
The heater relay is connected to pin RD0 through a transistor switch
The LM35DZ temperature sensor is connected to pin AN0 (RA0)

The program uses the built-in keypad library functions.

The control action is taken every 5 seconds.

Author:   Dogan Ibrahim
Date:     September 2013
File:     MIKROC-ON-OFF.C
*******************************************************************************/
// LCD module connections
sbit LCD_RS at RB4_bit;
sbit LCD_EN at RB5_bit;
sbit LCD_D4 at RB0_bit;
sbit LCD_D5 at RB1_bit;
sbit LCD_D6 at RB2_bit;
sbit LCD_D7 at RB3_bit;

sbit LCD_RS_Direction at TRISB4_bit;
sbit LCD_EN_Direction at TRISB5_bit;
sbit LCD_D4_Direction at TRISB0_bit;
sbit LCD_D5_Direction at TRISB1_bit;
sbit LCD_D6_Direction at TRISB2_bit;
sbit LCD_D7_Direction at TRISB3_bit;
// End LCD module connections

// Keypad module connections
char keypadPort at PORTC;
// End of keypad module connections

#define BUZZER PORTE.RE1
#define HEATER PORTD.RD0
#define Enter 12
#define ON 1
#define OFF 0
//
// Start of MAIN program
//
void main()
{
     unsigned char MyKey,Txt[14];
     unsigned int SetPoint;
     unsigned char *op;
     unsigned int temp;
     float mV, PlantTemp;
     
     ANSELB = 0;                                        	// Configure PORTB as digital
     ANSELC = 0;                                        	// Configure PORTC as digital
     ANSELD = 0;                                              	// Configure PORTD as digital
     ANSELE = 0;                                                // Configure PORTE as digital
     TRISA0_bit = 1;                                            // Configure AN0 (RA0) as input
     TRISB = 0;                              			// PORTB are outputs (LCD)
     TRISC = 0xF0;                           			// RC4-RC7 are inputs
     TRISD0_bit = 0;                                            // RD0 is output
     TRISE1_bit = 0;                                            // RE1 is output

     Keypad_Init();                                      	// Initialize keypad library
     Sound_Init(&PORTE,1);                                     	// Initialize Sound library
//
// Configure LCD
//
       Lcd_Init();                     		       		// Initialize LCD
       Lcd_Cmd(_LCD_CLEAR);
       Lcd_Out(1,1,"ON-OFF CONTROL");            	       	// Display CALCULATOR
       Delay_ms(2000);                       			// Wait 2 seconds
       Lcd_Cmd(_LCD_CLEAR);                   			// Clear display
       
       BUZZER = OFF;                                   		// TURN OFF buzzer to start with
       HEATER = OFF;                                   		// Turn OFF heater to start with
//
// On startup read the set point temperature from the keypad
//
       Lcd_Out(1,1,"Enter Set Point");
       setPoint = 0;
       
       Lcd_Out(2,1,"SP: ");
       while(1)                             			// Get first no
       {
          do
               MyKey = Keypad_Key_Click();
          while(!MyKey);
          if(MyKey == Enter)break;          			// If ENTER pressed
          if(MyKey == 10)MyKey = 0;				// If 0 key pressed
          Lcd_Chr_Cp(MyKey + '0');
          SetPoint = 10*SetPoint + MyKey;             		// First number in Op1
        }
        
        Lcd_cmd(_LCD_CLEAR);                             	// Clear LCD
        Lcd_Out(1,1,"SP = ");                           	// Display SP=
        IntToStr(SetPoint,Txt);                          	// Convert to string
        op = Ltrim(Txt);					// Remove leading spaces
        Lcd_Out_CP(op);
        Lcd_Out(2,1,"ENTER to cont.");
//
// Wait until ENTER is pressed
//      
        MyKey = 0;
        while(MyKey != Enter)                             	
        {
          do
               MyKey = Keypad_Key_Click();
          while(!MyKey);
        }
       

// Program loop
//
     for(;;)                                 			// Endless loop
     {
//
// Display the SetPoint and the Actual temperatures
//
        temp = ADC_Read(0);                         		// Read from AN0 (RA0)
        mV = temp*5000.0/1024.0;                    		// Convert to mV
        PlantTemp = mV / 10.0;                             	// Convert to degrees C
        Lcd_Cmd(_LCD_CLEAR);
        IntToStr(SetPoint,Txt);                          	// Convert to string
        op = Ltrim(Txt);
        Lcd_Out(1,1,"SP=");                              	// Display SP=
        Lcd_Out_Cp(op);                                    	// Display Set Point
        Lcd_Chr_Cp(178);
        Lcd_Chr_Cp('C');                          		// Display C
        Lcd_Out(2,1,"AC=");                        		// Display AC=
        FloatToStr(PlantTemp, Txt);                       	// Convert to string
        Lcd_Out_CP(Txt);
        Lcd_Chr_Cp(178);                      			// Display Degree sign
        Lcd_Chr_Cp('C');                      			// Display C character
//
// Implement the ON-OFF controller algorithm
//
        if(SetPoint > PlantTemp)                           	// If SetPoint is bigger than actual
        {
            HEATER = ON;                              		// Turn ON heater
            if((SetPoint - PlantTemp) > 2.0)          		// If outseide range
                Sound_Play(1000,1000);                          // Turn ON BUZZER
        }
        else                                         		// If Set Point is not bigger than actual
        {
            HEATER = OFF;                            		// Turn OFF heater
            if((PlantTemp - SetPoint) > 2.0)        		// Actual temp is bigger than setPoint
                Sound_Play(1000,1000);                         	// Turn ON BUZZER
        }  
        Delay_Ms(5000);                                         // Wait 5 seconds and repeat
     }
}

