/******************************************************************************
                           	Reaction Timer
                           	==============

This project is a reaction timer. Basically an LED is connected to RC0, a push-button switch
is connected to RC7 and an LCD is connected to PORTB. 

The LED turns ON after a random delay (between 1 and 10 seconds) and the user is expected
to press the button as soon as he/she sees the LED. The elapsed time between the LED  turning
ON and the button being pressed is displayed in milliseconds on the LCD

The project can measure reaction times from 1ms to just over 8.3 seconds (In practice it is not
possible for any person to react in more than a few seconds). A message is sent to the LCD if
the maximum reaction time is exceeded.

Programmer:     Dogan Ibrahim
Date:           September 2013
File:           MIKROC-REACT.C

*******************************************************************************/
// LCD module connections
sbit LCD_RS at RB4_bit;
sbit LCD_EN at RB5_bit;
sbit LCD_D4 at RB0_bit;
sbit LCD_D5 at RB1_bit;
sbit LCD_D6 at RB2_bit;
sbit LCD_D7 at RB3_bit;

sbit LCD_RS_Direction at TRISB4_bit;
sbit LCD_EN_Direction at TRISB5_bit;
sbit LCD_D4_Direction at TRISB0_bit;
sbit LCD_D5_Direction at TRISB1_bit;
sbit LCD_D6_Direction at TRISB2_bit;
sbit LCD_D7_Direction at TRISB3_bit;
// End LCD module connections

#define LED PORTC.RC0
#define SWITCH PORTC.RC7

void main()
{
     unsigned char T_Low, T_High, Txt[11];
     unsigned int No;
     unsigned long Cnt;
     
     ANSELB = 0;                                         // Configure PORTB as digital
     ANSELC = 0;                                         // Configure PORTC as digital
     TRISC.RC0 = 0;                                      // Configure RC0 as output
     TRISC.RC7 = 1;                                      // Configure RC7 as input
     
     Lcd_Init();                                         // Initialize LCD
     Lcd_Cmd(_LCD_CURSOR_OFF);                           //  Disable cursor
     T0CON = 0x07;                                       // Timer0, 16-bit, prescaler=256
     LED = 0;                                            // Turn OFF LED to start with
     srand(10);                                          // Initialize random number seed
     INTCON.TMR0IF = 0;                                  // Clear timer overflow flag
     
     for(;;)
     {
//
// Generate a random number between 1 and 32767 and chenge it to be between 1 and 10
//
       No =rand();                                        // Random number between 1 and 32767
       No = No % 10 + 1;                                  // Number between 1 and 10
       Vdelay_Ms(1000*No);                                // Delay No seconds
       TMR0H = 0;                                         // Clear Timer0 H register
       TMR0L = 0;                                         // Clear Timer0 L register
       LED = 1;                                           // Turn ON LED
//
// Turn ON Timer0 (counts in multiples of 128 microseconds
//
       T0CON.TMR0ON = 1;                                  // Turn ON Timer0
       while(SWITCH == 1);                                // Wait until the switch is pressed
//
// Switch is pressed. Stop timer and get the count
//
       T0CON.TMR0ON = 0;                                  // Stop Timer0
       T_Low = TMR0L;                                     // Read timer low byte
       T_High = TMR0H;                                    // read timer high byte
       Cnt = (unsigned long)256*T_High + T_Low;           // Get timer count
       Cnt = 128*Cnt / 1000;                              // Convert to milliseconds
       
       if(INTCON.TMR0IF == 1)                          	  // If timer overflow detected
       {
         Lcd_Out(1,1,"Too long...");
         Lcd_Out(2,1,"Try again...");
         INTCON.TMR0IF = 0;                               // Clear timer overflow flag
       }
       else
       {
//
// Convert count to string and display on LCD
//
          LongWordToStr(Cnt, Txt);                         // Convert to string
          Lcd_Out(1,1,"Reaction (ms)");                    // Display heading
          Lcd_Out(2,1,Txt);                                // Display reaction time in ms
        }
        LED = 0;                                           // Turn OFF LED
        Delay_ms(2000);                                    // Wait 2 seconds and repeat
        Lcd_Cmd(_LCD_CLEAR);                               // Clear LCD
     }
}
