/******************************************************************************
                  		THERMOMETER WITH RS232 SERIAL OUTPUT
                  		====================================

In this project a PC is connected to a PIC18F452K2 microcontroller. The project reads the ambient
temperature using an LM35DZ type analog temperature sensor. The temperature is then sent to
a PC over the RS232 communications line. The PC displays the temperature on the screen.

This program uses the built in USART of the microcontroller. The USART is configured to operate
with 9600 Baud rate.

The serial TX pin is RC6 and the serial RX pin is RC7.

Author:   Dogan Ibrahim
Date:     September 2013
File:     MIKROC-RS232-1.C
*******************************************************************************/

//
// This functions send carriage-return and line-feed to USART
//
void Newline()
{
    Uart1_Write(0x0D);                        		// Send carriage-return
    Uart1_Write(0x0A);                        		// Send line-feed
}


//
// Start of MAIN program
//
void main()
{
     unsigned char Txt[14];
     unsigned char msg1[] = "AMBIENT TEMPERATURE";
     unsigned char msg2[] = "===================";
     unsigned int temp;
     float mV;
     char *res;
     
     ANSELA = 1;					// Configure RA0 as analog
     ANSELC = 0;					// Configure PORTC as digital
     TRISA = 1;						// Configure RA0 as input

     Uart1_Init(9600);                           	// Initialize UART to Baud=9600
    
     Uart1_Write_Text(msg1);                         	// Display heading
     Newline();                                   	// Newline
     Uart1_Write_Text(msg2);
     Newline();
     
     for(;;)                                    	// Endless loop
     {

        temp = ADC_Read(0);                     	// Read temperature from channel 0
        mV = temp*5000.0 / 1024;                	// Convert to millivolts
        mV = mV / 10.0;                         	// Convert to Degrees Centigrade
        FloatToStr(mV, Txt);				// Convert into a string
        res = strchr(Txt, '.');                 	// Locate "."
        *(res+3) = '\0';                        	// Insert NULL character 2 digits after "."
        Uart1_Write_Text(Txt);				// Send temperature over the serial line
        Newline();
        Delay_Ms(1000);                         	// Wait 1 second
     }
}

