/*******************************************************************************
                     	SD CARD PROJECT - WRITE TEXT TO A FILE
                     	==================================

In this project an SD card is connected to PORT C as follows:

   CS    RC2
   CLK   RC3
   DO    RC4
   DI    RC5

The program creates a new file called MYFILE55.TXT on the SD card and writes the text
"This is MYFILE.TXT." to the file. Then the string "This is the added data..." is appended to the file.

Author:  Dogan Ibrahim
Date:    October 2013
File:    MIKROC-SD1.C
*******************************************************************************/
// MMC module connections
sbit Mmc_Chip_Select at LATC2_bit;
sbit Mmc_Chip_Select_Direction at TRISC2_bit;
// MMC module connections

#define FILE_READ 0x01         				// read only
#define FILE_WRITE 0x02        				// write only
#define FILE_APPEND 0x04       				// append to file

char filename[] = "MYFILE55.TXT";
unsigned char txt[] = "This is the added data...";
unsigned short character;
unsigned long file_size,i;

void main()
{   
    unsigned char fhandle;
    
    ANSELC = 0;                				// Configure PORTC as digital
//
// Initialise the SPI bus
//                    
    SPI1_Init_Advanced(_SPI_MASTER_OSC_DIV64, _SPI_DATA_SAMPLE_MIDDLE,
                       _SPI_CLK_IDLE_LOW, _SPI_LOW_2_HIGH);
//
// Initialize the Mmc library. Wait until card detected
//     
    while(Mmc_Fat_Init());
//
// Create new file (if it does not exist)
//
   fhandle = MMc_Fat_Open(&filename,FILE_WRITE,0x80);
//
// Clear the file, start at the beginning for writing
//
   Mmc_Fat_Rewrite();
//
// Write to the file, specifying the length of the text
//
   Mmc_Fat_Write("This is MYFILE.TXT.",19);
//
// Add more data to the end...
//
   Mmc_Fat_Append();
   Mmc_Fat_Write(txt,sizeof(txt));
//
// Now close the file (releases the handle)
//
   Mmc_Fat_Close();

   while(1);                               			// Wait here forever
}

