/******************************************************************************
                           	Digital Thermometer
                           	==================

This project is a digital thermometer with LCD display. An LM35DZ type analog temperature
sensor is used to sense the temperature. The sensor is connected to analog input AN0 (RA0)
of a PIC18F45K22 microcontroller operating with an 8MHz crystal. The program reads the temperature every second and displays on the LCD.

Programmer:     Dogan Ibrahim
Date:           September 2013
File:           MIKROC-THERMO.C

*******************************************************************************/
// LCD module connections
sbit LCD_RS at RB4_bit;
sbit LCD_EN at RB5_bit;
sbit LCD_D4 at RB0_bit;
sbit LCD_D5 at RB1_bit;
sbit LCD_D6 at RB2_bit;
sbit LCD_D7 at RB3_bit;

sbit LCD_RS_Direction at TRISB4_bit;
sbit LCD_EN_Direction at TRISB5_bit;
sbit LCD_D4_Direction at TRISB0_bit;
sbit LCD_D5_Direction at TRISB1_bit;
sbit LCD_D6_Direction at TRISB2_bit;
sbit LCD_D7_Direction at TRISB3_bit;
// End LCD module connections

void main()
{
     unsigned char Txt[14];
     unsigned int temp;
     float mV;
     
     ANSELA = 0;                                         // Configure PORTA as digital
     ANSELB = 0;                                         // Configure PORTB as digital
     TRISA.RA0 = 1;                                      // RA0 is input
     Lcd_Init();                                         // Initialize LCD
     
     Lcd_Cmd(_LCD_CURSOR_OFF);                           //  Disable cursor
     
     for(;;)
     {
       temp = ADc_Read(0);                                // Read from channel 0
       mV = temp*5000.0 / 1024.0;                         // Convert to mV
       mV = mV / 10.0;                                    // mV is now the Degrees C
       floatToStr(mV, Txt);                               // Convert to string
       Lcd_cmd(_LCD_CLEAR);                               // Clear LCD
       Lcd_out(1,1,"Temperature");                        // Display heading
       Lcd_Out(2,1,Txt);                                  // Display temperature
       Delay_ms(1000);                                    // Wait 1 second and repeat
     }
}
