/******************************************************************************
                           	UNIPOLAR STEPPER MOTOR PROJECT
                           	==============================

In this project an UAG2 model unipolar stepper motor is connected to PORTB of a PIC18F45K22
microcontroller through IRLI520N type MOSFET transistor switches.

The program rotates the motor 100 times and then stops. In total 2000 pulses are sent to the
motor. 3ms delay is inserted between each pulse. Therefore, the motor rotates 100 revolutions
in 6 seconds (2000x3ms = 6s) and then stops.

Author:   Dogan Ibrahim
Date:     October 2013
File:     MIKROC-USTP1.C
*******************************************************************************/
const unsigned int Req_Rev_Count = 100;                   	// Required no of revs
const unsigned char Step_Size = 18;                   		// Motor Step Size (degrees)

void main()
{
     unsigned char Step[4] = {1, 2, 4, 8};
     unsigned int One_Rev_Step, Step_Count, Cycle_Count, j;
     unsigned char i;
     
     ANSELB = 0;                                      		// Configure PORTB as digital
     TRISB = 0;                                        		// Configure PORTB as digital
     
     One_Rev_Step = 360/Step_Size;                    		// No of steps for 1 revolution
     Step_Count = Req_Rev_Count*One_Rev_Step;              	// Total no of steps required
     Cycle_Count = Step_Count/4;                       		// No of cycles
     
     for(j = 0; j < Cycle_Count; j++)                 		// Do for all cycles
     {
        for(i = 0; i < 4; i++)                      		// Do for all steps
        {
           PORTB = Step[i];                         		// Send pulses to the motor
           Delay_Ms(3);                              		// 3 ms delay between each pulse
        }
     }
     while(1);                                			// End. Wait here forever
}
