/******************************************************************************
	       		Sawtooth Waveform Generation
	        	===========================

This project shows how a sawtooth wavefor with specified frequency can be generated using a microcontroller. The PIC18F45K22 microcontroller is used with an 8MHz crystal.

The generated sawtooth waveform has amplitude 0V to +5V and frequency of 100Hz (period: 10ms).

The MCP4921 DAC chip is used to convert the generated signal into analog.  This is a 12-bit
converter controlled with the SPI bus signals.

Programmer:     Dogan Ibrahim
Date:           September 2013
File:           MIKROC-WAVE1.C

*******************************************************************************/
// DAC module connections
sbit Chip_Select at RC0_bit;
sbit Chip_Select_Direction at TRISC0_bit;
// End DAC module connections

//
// This function sends 12 bits digital data to the DAC. The data is passed
// through the function argument called "value"
//
void DAC(unsigned int value)
{
  char temp;

  Chip_Select = 0;                       		// Enable DAC chip

  // Send High Byte
  temp = (value >> 8) & 0x0F;            		// Store bits 8:11 to temp
  temp |= 0x30;                          		// Define DAC setting (choose 1x gain)
  SPI1_Write(temp);                      		// Send high byte via SPI

  // Send Low Byte
  temp = value;                       			// Store bits 0:7 to temp
  SPI1_Write(temp);                      		// Send low byte via SPI

  Chip_Select = 1;                       		// Disable DAC chip
}



void main()
{
     float i;
     unsigned int DAC_Value;

     ANSELC = 0;                         		// Configure PORTC as digital
     Chip_select = 1;                    		// Disable DAC
     Chip_Select_Direction = 0;          		// Set CS as output
     SPI1_Init();                        		// Initialize SPI module
 //
 // Generate the Sawtooth waveform
 //
     for(;;)                             		// Endless loop
     {
        for(i = 0; i <= 1; i = i + 0.1)       		// Generate waveform steps
        {
           DAC_Value = i * 4095;
           DAC(DAC_Value);                   		// Send to DAC converter
           Delay_us(909);                     		// Wait 909 microseconds
        }
     }
}
