/******************************************************************************
                        		BIPOLAR STEPPER MOTOR ROTATION
                       		 ==============================

In this project a 39HS model bipolar stepper motor is connected to PORTB of a PIC18F45K22
microcontroller through an A3967SLB type bipolar motor controller chip

The program rotates the motor clockwise 10 revolutions, stops for 5 seconds, and then
rotates anticlockwise for 10 turns and stops.

39HS motor has step angle of 1.8 degrees. thus, a complete revolution requires 200 pulses
(steps) to be sent to the motor. For 10 revolutions, we have to send 2000 pulses. The DIR
input controls the direction of rotation.

Author:   Dogan Ibrahim
Date:     October 2013
File:     XC8-BSTP.C
*******************************************************************************/
#include <xc.h>
#pragma config MCLRE = EXTMCLR, WDTEN = OFF, FOSC = HSHP
#define _XTAL_FREQ 8000000

#define ENABLE PORTBbits.RB0                               	// A3967SLB ENABLE input
#define STEP PORTBbits.RB1                          		// A3967SLB STEP input
#define RST PORTBbits.RB2                         		// A3967SLB RESET input
#define SLP PORTBbits.RB3                          		// A3967SLB SLEEP input
#define MS1 PORTBbits.RB4                            		// A3967SLB MS1 input
#define MS2 PORTBbits.RB5                         		// A3967SLB MS2 input
#define DIR PORTBbits.RB6                          		// A3967SLB DIR input

//
// This function creates seconds delay. The argument specifies the delay time in seconds
//
void Delay_Seconds(unsigned char s)
{
    unsigned char i,j;

    for(j = 0; j < s; j++)
    {
        for(i = 0; i <  100; i++)__delay_ms(10);
    }
}

//
// This is the main program. The motor turns 10 revolutins clockwise with 1ms delay
// between each step. Then the motor stops for 5 seconds. The motor then rotates
// anticlockwise for 10 revolutions and hen stops
//
void main()
{
     unsigned int i;

     ANSELB = 0;                                      			// Configure PORTB as digital
     TRISB = 0;                                      			// Configure PORTB as digital
//
// A3967SLB chip configuration
//
     ENABLE = 0;                                     			// Enable the controller chip
     RST = 1;                                      			// Disable RESET
     SLP = 1;                                       			// Disable SLEEP
     MS1 = 0;                                         			// Full-step sequence
     MS2 = 0;
     STEP = 0;                                         			// Step = 0 to start with
//
// First rotate clockwise 10 revolutions
//
     DIR = 0;
     for(i = 0; i < 2000; i++)
     {
        STEP = 1;                                     			// Send STEP pulses
        STEP = 0;
        __delay_ms(1);
     }

     Delay_Seconds(5);          					// Wait 5 seconds
//
// Now rotate anticlockwise 10 revolutions
//
    DIR = 1;
    for(i = 0; i < 2000; i++)
    {
       STEP = 1;                                   			// Send STEP pulses
       STEP = 0;
       __delay_ms(1);
    }
    while(1);            						// End. Wait here forever
}
