/*******************************************************************************
                  		CALCULATOR WITH KEYPAD AND LCD
                  		==============================

In this project a 4 x 4 keypad is connected to PORTC of a PIC18F45K22 microcontroller. Also an
LCD is connected to PORTB. The project is a simple calculator which can perform integer arithmetic.

 he LCD is connected to the microcontroller as follows:

Microcontroller      	LCD
============        	===

    RB0              	D4
    RB1              	D5
    RB2              	D6
    RB3              	D7
    RB4              	E
    RB5              	R/S
    RB6              	RW

The keys are organised as follows:

    0     1     2     3
    4     5     6     7
    8     9    10    11
    12   13    14    15

The keys are labelled as follows:

    1   2   3   4
    5   6   7   8
    9   0       Enter
    +   -   *   /


Author:   Dogan Ibrahim
Date:     August 2013
File:     XC8-KEYPAD.C
*******************************************************************/

#include <xc.h>
#include <stdlib.h>
#include <plib/xlcd.h>
#include <plib/delays.h>
#pragma config MCLRE = EXTMCLR, WDTEN = OFF, FOSC = HSHP
#define _XTAL_FREQ 8000000

#define MASK 0xF0
#define Enter 11
#define Plus 12
#define Minus 13
#define Multiply 14
#define Divide 15


//
// This function creates seconds delay. The argument specifies the delay time in seconds
//
void Delay_Seconds(unsigned char s)
{
    unsigned char i,j;

    for(j = 0; j < s; j++)
    {
        for(i = 0; i <  100; i++)__delay_ms(10);
    }
}

//
// This function creates 18 cycles delay for the xlcd library
//
void DelayFor18TCY( void )
{
Nop(); Nop(); Nop(); Nop();
Nop(); Nop(); Nop(); Nop();
Nop(); Nop(); Nop(); Nop();
Nop(); Nop();
return;
}


//
// This fucntion creats 15ms delay for the xlcd library
//
void DelayPORXLCD( void )
{
    __delay_ms(15);
    return;
}


//
// This function creates 5ms delay for the xlcd library
//
void DelayXLCD( void )
{
    __delay_ms(5);
    return;
}

//
// This function clears the screen
//
void LCD_Clear()
{
    while(BusyXLCD());
    WriteCmdXLCD(0x01);
}

//
// This function moves the cursor to position row,column
//
void LCD_Move(unsigned char row, unsigned char column)
{
  char ddaddr = 40*(row-1) + column;
  while( BusyXLCD() );
  SetDDRamAddr( ddaddr );
}


//
// This function gets a key from the keypad and returns it to calling program
//
unsigned char getkeypad()
{
        unsigned char i, Key = 0;

        PORTC = 0x01;                        			// Start with column 1
        while((PORTC & MASK) == 0)           	        	// While no key pressed
        {
             PORTC = (PORTC << 1);           			// next column
             Key++;                          			// column number
             if(Key == 4)
             {
                 PORTC = 0x01;               			// Back to column 1
                 Key = 0;
             }
        }
        __delay_ms(20);                        			// Switch debounce

        for(i = 0x10; i !=0; i <<=1)         	        	// Find the key pressed
        {
           if((PORTC & i) != 0)break;
           Key = Key + 4;
        }

        PORTC = 0x0F;
        while((PORTC & MASK) != 0);             		// Wait until key released
        __delay_ms(20);                        			// Switch debounce

        return (Key);                          			// Return key number
}



//
// Start of MAIN program
//
void main()
{
     unsigned char MyKey, i,j,op[10];
     unsigned long Calc, Op1, Op2;

     ANSELB = 0;                                        	// Configure PORTB as digital
     ANSELC = 0;                                        	// Configure PORTC as digital
     TRISB = 0;                              			// PORTB are outputs (LCD)
     TRISC = 0xF0;                           			// RC4-RC7 are inputs

     OpenXLCD(FOUR_BIT & LINES_5X7);                   		// Initialize LCD
     while(BusyXLCD());                     			// Wait if the LCD is busy
     WriteCmdXLCD(DON);                     			// Turn Display ON
     while(BusyXLCD());                     			// Wait if the LCD is busy
     WriteCmdXLCD(0x06);                    			// Move cursor right
     putrsXLCD("CALCULATOR");                			// Display heading
     Delay_Seconds(2);                      			// 2 seconds delay
     LCD_Clear();                           			// Clear display
//
// Program loop
//
     for(;;)                                 			// Endless loop
     {
        MyKey = 0;
        Op1 = 0;
        Op2 = 0;

        LCD_Move(1,1);                                  	// Move to row=1,column=1
        putrsXLCD("No1: ");                			// Display No1:
        while(1)                             			// Get first no
        {
           MyKey = getkeypad();
           if(MyKey == Enter)break;          			// If ENTER pressed
           MyKey++;                                     	// Key number pressed
           if(MyKey == 10)MyKey = 0;				// If 0 key pressed
           putcXLCD(MyKey + '0');
           Op1 = 10*Op1 + MyKey;             			// First number in Op1
        }

        LCD_Move(2,1);                                  	// Move to row=2,column=1
        putrsXLCD("No2: ");                			// Display No2:
        while(1)                             			// Get second no
        {
           MyKey = getkeypad();
           if(MyKey == Enter)break;          			// If ENTER pressed
           MyKey++;
           if(MyKey == 10)MyKey = 0;				// If 0 key pressed
           putcXLCD(MyKey + '0');
           Op2 = 10*Op2 + MyKey;             			// Second number in Op2
        }

        LCD_Clear();                  				// Clear LCD
        LCD_Move(1,1);                                      	// Move to row=1,column=1
        putrsXLCD("Op: ");                 			// Display Op:

        MyKey = getkeypad();                 			// Get operation
        LCD_Clear();
        LCD_Move(1,1);
        putrsXLCD("Res=");                 			// Display Res=
        switch(MyKey)                        			// Perform the operation
        {
          case Plus:
                  Calc = Op1 + Op2;          			// If ADD
                  break;
          case Minus:
                  Calc = Op1 - Op2;          			// If Subtract
                  break;
          case Multiply:
                  Calc = Op1 * Op2;          			// If Multiply
                  break;
          case Divid
                  Calc = Op1 / Op2;          			// If Divide
                  break;
        }

        ltoa(op, Calc, 10);                 			// Convert to string in op
        putsXLCD(op);                     			// Display result
        Delay_Seconds(5);                      			// Wait 5 seconds
        LCD_Clear();
     }
}
