/*******************************************************************************
                  		VOLTMETER WITH LCD DISPLAY
                  		==========================

In this project an LCD is connected to PORTB. Also, input port AN0 is used as analog input. 
Voltage to be measured is applied to AN0. The microcontroller reads the analog voltage, 
converts into digital, and then displays on the LCD.

Analog input range is 0 to 5V. A PIC18F45K22 type microcontroller is used in this project,
operated with an 8MHz crystal.

Analog data is read using the Adc_Read built-in function.

The LCD is connected to the microcontroller as follows:

Microcontroller      	LCD
============        	 ===

    RB0              	D4
    RB1              	D5
    RB2              	D6
    RB3              	D7
    RB4              	E
    RB5              	R/S
    RB6              	RW

Author:   	Dogan Ibrahim
Date:     	August 2013
File:     	XC8-LCD1.C
**************************************************************/

#include <xc.h>
#include <stdlib.h>
#include <plib/adc.h>
#include <plib/xlcd.h>
#include <plib/delays.h>
#pragma config MCLRE = EXTMCLR, WDTEN = OFF, FOSC = HSHP
#define _XTAL_FREQ 8000000

//
// This function creates seconds delay. The argument specifies the delay time in seconds
//
void Delay_Seconds(unsigned char s)
{
    unsigned char i,j;

    for(j = 0; j < s; j++)
    {
        for(i = 0; i <  100; i++)__delay_ms(10);
    }
}

//
// This function creates 18 cycles delay for the xlcd library
//
void DelayFor18TCY( void )
{
Nop(); Nop(); Nop(); Nop();
Nop(); Nop(); Nop(); Nop();
Nop(); Nop(); Nop(); Nop();
Nop(); Nop();
return;
}


//
// This fucntion creats 15ms delay for the xlcd library
//
void DelayPORXLCD( void )
{
    __delay_ms(15);
    return;
}


//
// This function creates 5ms delay for the xlcd library
//
void DelayXLCD( void )
{
    __delay_ms(5);
    return;
}

//
// This function clears the screen
//
void LCD_Clear()
{
    while(BusyXLCD());
    WriteCmdXLCD(0x01);
}

//
// This function moves the cursor to position row,column
//
void LCD_Move(unsigned char row, unsigned char column)
{
  char ddaddr = 40*(row-1) + column;
  while( BusyXLCD() );
  SetDDRamAddr( ddaddr );
}


void main()
{
     unsigned long Vin, mV;
     char op[10];
    
     ANSELB = 0;                                   		// Configure PORTB as digital
     ANSELA = 1;                                   		// COnfigure RA0 as analog
     TRISB = 0;                       		   		// Configure PORTB as outputs
     TRISA = 1;                    		   		// Configure RA0 as input
//
// Configure the LCD to use 4-bits, in multiple display mode
//
     Delay_Seconds(1);
     OpenXLCD(FOUR_BIT & LINES_5X7);
//
// Configure the A/D converter
//
    OpenADC(ADC_FOSC_2 & ADC_RIGHT_JUST & ADC_20_TAD,
                       ADC_CH0 & ADC_INT_OFF,
                       ADC_TRIG_CTMU & ADC_REF_VDD_VDD & ADC_REF_VDD_VSS);

     while(BusyXLCD());                     			// Wait if the LCD is busy
     WriteCmdXLCD(DON);                     			// Turn Display ON
     while(BusyXLCD());                     			// Wait if the LCD is busy
     WriteCmdXLCD(0x06);                    			// Move cursor right
     putrsXLCD("VOLTMETER");                			// Display heading
     Delay_Seconds(2);                      			// 2 seconds delay
     LCD_Clear();                           			// Clear display

     for(;;)                          		   		// Endless loop
     {
        LCD_Clear();                              		// Clear LCD
        SelChanConvADC(ADC_CH0);                   		// Select channel 0 and start conversion
        while(BusyADC());                          		// Wait for completion
        Vin = ReadADC();                           		// Read converted data
        mV = (Vin * 5000) >> 10;      		   		// mv = Vin x 5000 / 1024
        LCD_Move(1,1);                              		// Move to row=1, column=1
        putrsXLCD("mV = ");         				// Display "mV = "
        mV = (Vin * 5000) >> 10;      				// mv = Vin x 5000 / 1024
        itoa(op, mV, 10);                           		// Convert mV into ASCII string
//
// Display result on LCD
//
        LCD_Move(1,6);                              		// Move to row=1,column=6
        putsXLCD(op);                               		// Display the measured voltage
        Delay_Seconds(1);               			// Wait 1 second
     }
}
