/******************************************************************************
                          	4-DIGIT 7-SEGMENT DISPLAY
                          	=========================

In this project four common cathode 7-segment LED displays are connected to PORTD
of a PIC18F45K22 microcontroller and the microcontroller is operated from an 8MHz crystal
Four PORTA pins are used to enable/disable the LEDs.

The program displays number 1234.


Author:           Dogan Ibrahim
Date:             August 2013
File:             XC8-LED10.C
*******************************************************************************/

#include <xc.h>
#pragma config MCLRE = EXTMCLR, WDTEN = OFF, FOSC = HSHP
#define _XTAL_FREQ 8000000

#define DIGIT1 PORTAbits.RA0
#define DIGIT2 PORTAbits.RA1
#define DIGIT3 PORTAbits.RA2
#define DIGIT4 PORTAbits.RA3

//
// This function finds the bit pattern to be sent to the port to display a number
// on the 7-segment LED. The number is passed in the argument list of the function.
//
unsigned char Display(unsigned char no)
{
    unsigned char Pattern;
    unsigned char SEGMENT[] = {0x3F,0x06,0x5B,0x4F,0x66,0x6D,0x7D,0x07,0x7F,0x6F};

    Pattern = SEGMENT[no];             			// Pattern to return
    return (Pattern);
}


void main()
{
     unsigned int Cnt = 1234;
     unsigned int D1,D2,D3,D4,D5,D6;
     ANSELA = 0;                                 		// Configure PORTA as digital
     ANSELD = 0;                                 		// Configure PORTD as digital
     TRISA = 0;                                  		// Configure PORTA as outputs
     TRISD = 0;                                  		// Configure PORTD as outputs

     DIGIT1 = 0;                                 		// Disable digit 1
     DIGIT2 = 0;                                 		// Disable digit 2
     DIGIT3 = 0;                                 		// Disable digit 3
     DIGIT4 = 0;                                 		// Disable digit 4

     for(;;)                                     		// Endless loop
     {
        D1 = Cnt / 1000;                         		// 1000s  digit
        PORTD = Display(D1);                     		// Send to PORTD
        DIGIT4 = 1;                              		// Enable digit 4
        __delay_ms(5);                             		// Wait a while
        DIGIT4 = 0;                              		// Disable digit 4

        D2 = Cnt % 1000;
        D3 = D2 / 100;                           		// 100s digit
        PORTD = Display(D3);                     		// Send to PORTD
        DIGIT3 = 1;                              		// Enable digit 3
        __delay_ms(5);                             		// Wait a while
        DIGIT3 = 0;                              		// Disable digit 3

        D4 = D2 % 100;
        D5 = D4/10;                              		// 10s digit
        PORTD = Display(D5);                     		// Send to PORTD
        DIGIT2 = 1;                              		// Enable digit 2
        __delay_ms(5);                                    	// Wait a while
        DIGIT2 = 0;                              		// Disable digit 2

        D6 = D4 % 10;
        PORTD = Display(D6);                     		// Send to PORTD
        DIGIT1 = 1;                              		// Enable digit 1
        __delay_ms(5);                             		// Wait a while
        DIGIT1 = 0;                                     	// Disable digit 1

     }
}
