/******************************************************************************
                  			VOLTMETER WITH LED DISPLAYS
                  			===========================

In this project 5 LEDs are connected to PORTD. Also, input port RA0 (AN0) is used as analog
input. Voltage to be measured is applied to this pin. The microcontroller reads the analog
voltage, converts into digital, and then turns ON one of the LEDs to indicate the voltage range.

Analog input range is 0 to 5V. A PIC18F45K22 type microcontroller is used in this project,
operated with an 8MHz cystal.

Analog data is read using the MPLAB XC8 built-in functions


Author:   	Dogan Ibrahim
Date:     	August 2013
File:     	XC8-LED11.C
*******************************************************************************/
#include <xc.h>
#include <plib/adc.h>
#pragma config MCLRE = EXTMCLR, WDTEN = OFF, FOSC = HSHP
#define _XTAL_FREQ 8000000

#define LED01V PORTDbits.RD0
#define LED12V PORTDbits.RD1
#define LED23V PORTDbits.RD2
#define LED34V PORTDbits.RD3
#define LED45V PORTDbits.RD4

void main()
{
     unsigned long Vin, mV;

     ANSELD = 0;                                   		// Configure PORTD as digital
     ANSELA = 1;                                   		// COnfigure RA0 as analog
     TRISD = 0;                       		   		// Configure PORTD as outputs
     TRISA = 1;                    		  		// Configure RA0 as input
//
// Configure A/D converter
//
    OpenADC(ADC_FOSC_2 & ADC_RIGHT_JUST & ADC_12_TAD,
            ADC_CH0 & ADC_INT_OFF,
            ADC_TRIG_CTMU & ADC_REF_VDD_VDD & ADC_REF_VDD_VSS);
    
     for(;;)                          		   		// Endless loop
     {
        PORTD = 0;                                 		// Clear LEDs to start with
        SelChanConvADC(ADC_CH0);                   		// Select channel 0 and start conversion
        while(BusyADC());                          		// Wait for completion
        Vin = ReadADC();                           		// Rea converted data
        mV = (Vin * 5000) >> 10;      		   		// mv = Vin x 5000 / 1024
//
// Find which LED to turn ON
//
        if(mV <= 1000)                             		// Between 1 - 2V
        {
           LED01V = 1;                             		// Turn ON LED0-1V
        }
        else if(mV > 1000 && mV <= 2000)           		// Between 2 - 3V
        {
           LED12V = 1;                             		// Turn ON LED1-2V
        }
        else if(mV > 2000 && mV <= 3000)           		// Between 2 - 3V
        {
          LED23V = 1;                              		// Turn ON LED2 -3V
        }
        else if(mV > 3000 && mV <= 4000)           		// Between 3 - 4V
        {
          LED34V = 1;                              		// Turn ON LED3-4V
        }
        else if(mV > 4000 && mV <= 5000)           		// Between 4 - 5V
        {
          LED45V = 1;                              		// Turn ON LED4-5V
        }
        __delay_ms(10);                            		// 10ms delay
     }
}
