/*****************************************************************************
                  		TWO DICE - LESS I/O COUNT
                  		=========================

In this project LEDs are connected to PORT C of a PIC18F452 microcontroller and the
microcontroller is operated from an 8MHz crystal. The LEDs are organised as the faces
of a real dice. When a push-button switch connected to RB0 is pressed a dice pattern
is displayed on the LEDs. The display remains in this state for 3 seconds and after this
period the LEDs all turn OFF to indicate that the system is ready for the button to be
pressed again.

In this program a pseudo random number generator function is used to generate
the dice numbers between 1 and 6.

Author:   	Dogan Ibrahim
Date:     	August 2013
File:     	XC8-LED7-1.C
**************************************************************************/

#include <xc.h>
#pragma config MCLRE = EXTMCLR, WDTEN = OFF, FOSC = HSHP
#define _XTAL_FREQ 8000000

#define Switch PORTBbits.RB0
#define Pressed 0

//
// This function creates seconds delay. The argument specifies the delay time
// in seconds
//
void Delay_Seconds(unsigned char s)
{
    unsigned char i,j;

    for(j = 0; j < s; j++)
    {
        for(i = 0; i <  100; i++)__delay_ms(10);
    }
}


void main()
{
      unsigned char Pattern, J = 1;
     unsigned char DICE[] = {0,0x88,0x18,0x98,0x68,0xE8,0x78,
                               0x81,0x11,0x91,0x61,0xE1,0x71,
                               0x89,0x19,0x99,0x69,0xE9,0x79,
                               0x86,0x16,0x96,0x66,0xE6,0x76,
                               0x8E,0x1E,0x9E,0x6E,0xEE,0x7E,
                               0x87,0x17,0x97,0x67,0xE7,0x77};

     ANSELC = 0;                            		// Configure PORTC as digital
     ANSELB = 0;                            		// Configure PORTB as digital
     TRISC = 0;                       	    		// Configure PORT C as outputs
     TRISB = 1;                       	    		// Configure RB0 as input
     PORTC = 0;                       	   		 // Turn OFF all LEDs

     for(;;)                          	    		// Endless loop
     {
          if(Switch == Pressed)       	    		// Is switch pressed ?
          {
            Pattern = DICE[J];        	    		// Number to send to PORTC
            PORTC = Pattern;          	    		// Send to PORT C
            Delay_Seconds(3);           	    	// 3 seconds delay
            PORTC = 0;                	    		// Clear PORT C
          }
            J++;                      	    		// Increment J
            if(J == 37) J = 1;        	    		// If J = 37, reset to 1
     }

}
