/*******************************************************************************
                  		TWO DICE - FEWER I/O COUNT
                  		==========================

In this project LEDs are connected to PORT C of a PIC18F45K22 microcontroller and the
microcontroller is operated from an 8MHz crystal. The LEDs are organised as the faces of a
real dice. When a push-button switch connected to RB0 is pressed a dice pattern is displayed
on the LEDs. The display remains in this state for 3 seconds and after this period the LEDs all
turn OFF to indicate that the system is ready for the button to be pressed again.

In this program a pseudo random number generator function is used to generate the dice
numbers between 1 and 6.

Author: 	Dogan Ibrahim
Date:     	August 2013
File:     	XC8-LED7.C
********************************************************************************/

#include <xc.h>
#include <stdlib.h>
#pragma config MCLRE = EXTMCLR, WDTEN = OFF, FOSC = HSHP
#define _XTAL_FREQ 8000000

#define Switch PORTBbits.RB0
#define Pressed 0

//
// This function creates seconds delay. The argument specifies the delay time in seconds
//
void Delay_Seconds(unsigned char s)
{
    unsigned char i,j;

    for(j = 0; j < s; j++)
    {
        for(i = 0; i <  100; i++)__delay_ms(10);
    }
}


void main()
{
     unsigned char J,L,U,R;
     unsigned char DICE[] = {0,0x08,0x22,0x2A,0x55,0x5D,0x77};

     ANSELC = 0;                                	// Configure PORTC as digital
     ANSELB = 0;                                	// Configure PORTB as digital
     TRISC = 0;		            	        	// Configure PORTC as outputs
     TRISB = 1;            		        	// Configure RB0 as input
     PORTC = 0;                       			// Turn OFF all LEDs
     srand(1);                                  	// Initialize rand function

     for(;;)                          			// Endless loop
     {
          if(Switch == Pressed)       			// Is switch pressed ?
          {
            J = rand() / 6553 + 1;              	// Generate a number 1 to 6
            L = DICE[J];                        	// Get dice pattern
            J = rand() / 6553 + 1;              	// Generate another number
            U = DICE[J];                        	// Get dice pattern
            R = 16*U + L;                       	// Bit pattern to send to PORTC
            PORTC = R;                          	// Send bit pattern to PORTC
            Delay_Seconds(3);           		// Delay 3 second
            PORTC = 0;                			// Turn OFF PORTC LEDs
           }
     }
}
