/******************************************************************************
                  		Dual 7-SEGMENT DISPLAY
                  		======================

In this project two common cathode 7-segment LED displays are connected to PORTD of a PIC18F45K22 microcontroller and the microcontroller is operated from an 8MHz crystal
Digit 1 (right digit) enable pin is connected to port pin RA0 and digit 2 (left digit) enable pin
is connected to port pin RA1 of the microcontroller. The program displays number 25 on the
displays.

Author:   	Dogan Ibrahim
Date:     	August 2013
File:     	XC8-LED9.C
*******************************************************************************/

#include <xc.h>
#pragma config MCLRE = EXTMCLR, WDTEN = OFF, FOSC = HSHP
#define _XTAL_FREQ 8000000

#define DIGIT1 PORTAbits.RA0
#define DIGIT2 PORTAbits.RA1

//
// This function finds the bit pattern to be sent to the port to display a number on the
// 7-segment LED. The number is passed in the argument list of the function.
//
unsigned char Display(unsigned char no)
{
    unsigned char Pattern;
    unsigned char SEGMENT[] = {0x3F,0x06,0x5B,0x4F,0x66,0x6D,
                               0x7D,0x07,0x7F,0x6F};

    Pattern = SEGMENT[no];             		// Pattern to return
    return (Pattern);
}


void main()
{
     unsigned char Msd, Lsd, Cnt = 25;

     ANSELA = 0;                                		// Configure PORTA as digital
     ANSELD = 0;                                		// Configure PORTD as digital
     TRISA = 0;                       				// Configure PORTA as outputs
     TRISD = 0;                       				// Configure PORTD as outputs

     DIGIT1 = 0;                      				// Disable digit 1
     DIGIT2 = 0;                      				// Disable digit 2

     for(;;)                          				// Endless loop
     {
        Msd = Cnt / 10;               				// MSD digit
        PORTD = Display(Msd);					// Send to PORTD
        DIGIT2 = 1;                   				// Enable digit 2
        __delay_ms(10);                 			// Wait a while

        DIGIT2 = 0;                   				// Disable digit 2
        Lsd = Cnt % 10;               				// LSD digit
        PORTD = Display(Lsd);         				// Send to PORTD
        DIGIT1 = 1;                   				// Enable digit 1
        __delay_ms(10);                 			// Wait a while
        DIGIT1 = 0;                   				// Disable digit 1
     }
}
