/******************************************************************************
                          THERMOMETER WITH RS232 SERIAL OUTPUT
                          ====================================

In this project a PC is connected to a PIC18F452K2 microcontroller. The project reads the
ambient temperature using an LM35DZ type analog temperature sensor. The temperature is
then sent to a PC over the RS232 communications line. The PC displays the temperature on the screen.

This program uses the built in USART of the microcontroller. The USART is configured to operate
with 9600 Baud rate.

The serial TX pin is RC6 and the serial RX pin is RC7.

Author:   Dogan Ibrahim
Date:     September 2013
File:     XC8-RS232-1.C
*******************************************************************************/
#include <xc.h>
#include <string.h>
#include <plib/usart.h>
#include <plib/adc.h>
//#include <strings.h>
#include <stdlib.h>
#pragma config MCLRE = EXTMCLR, WDTEN = OFF, FOSC = HSHP
#define _XTAL_FREQ 8000000

//
// This function creates seconds delay. The argument specifies the delay time in seconds
//
void Delay_Seconds(unsigned char s)
{
    unsigned char i,j;

    for(j = 0; j < s; j++)
    {
        for(i = 0; i <  100; i++)__delay_ms(10);
    }
}


void Newline()
{
    Write1USART(0x0D);                                        	// Send carriage-return
    while(Busy1USART());                                	// Wait while UART is busy
    Write1USART(0x0A);                                        	// Send line-feed
}


//
// Start of MAIN program
//
void main()
{
     char *Txt;
     unsigned char msg1[] = "AMBIENT TEMPERATURE";
     unsigned char msg2[] = "===================";
     unsigned int temp;
     float mV;
     char *res;
     int status;

     ANSELA = 1;                    				// Configure RA0 as analog
     ANSELC = 0;                    				// COnfigure PORTC as digital
     TRISA = 1;                     				// Configure RA0 as input

     Open1USART( USART_TX_INT_OFF	&
                USART_RX_INT_OFF	&
                USART_ASYNCH_MODE	&
                USART_EIGHT_BIT 	&
                USART_CONT_RX 		&
                USART_BRGH_LOW,
                12);

     putrs1USART(msg1);                         		// Display heading
     Newline();                                   		// Newline
     putrs1USART(msg2);
     Newline();
//
// Configure A/D converter
//
    OpenADC(ADC_FOSC_2 & ADC_RIGHT_JUST & ADC_12_TAD,
            ADC_CH0 & ADC_INT_OFF,
            ADC_TRIG_CTMU & ADC_REF_VDD_VDD & ADC_REF_VDD_VSS);


     for(;;)                                    		// Endless loop
     {
        SelChanConvADC(ADC_CH0);                  		// Select channel 0 and start conversion
        while(BusyADC());                          		// Wait for completion
        temp = ReadADC();                           		// Read converted data
        mV = temp * 5000.0 / 1024;      		   	// Convert to mV
        mV = mV / 10.0;                         		// Convert to Degrees Centigrade
        Txt = ftoa(mV, &status);                		// Convert to string
        
        res = strchr(Txt, '.');                 		// Locate "."
        *(res+3) = '\0';                        		// Insert NULL character 2 digits after "."
        putrs1USART(Txt);
        Newline();
        Delay_Seconds(1);                         		// Wait 1 second
     }
}
