/*******************************************************************************
                          	PLAYING MELODY
                         	==============

In this project a buzzer is connected to port pin RC2 of a PIC18F45K22 microcontroller, 
operating with an 8MHz crystal.

The program plays the classical "Happy Birthday" melody.


Author:           Dogan Ibrahim
Date:             August 2013
File:             XC8-SOUND1.C
**********************************************************************************/

#include <xc.h>
#pragma config MCLRE = EXTMCLR, WDTEN = OFF, FOSC = HSHP
#define _XTAL_FREQ 8000000
#define BUZZER LATCbits.LATC2
#define Max_Notes 25

unsigned char TIMER_H, TIMER_L;

unsigned int Notes[Max_Notes] =
{
  262, 262, 294, 262, 349, 330, 262, 262, 294, 262, 392,
  349, 262, 262, 524, 440, 349, 330, 294, 466, 466, 440,
  349, 392, 349
};

unsigned char Durations[Max_Notes] =
{
  1, 1, 2, 2, 2, 3, 1, 1, 2, 2, 2, 3, 1, 1, 2, 2, 2, 2, 2,
  1, 1, 2, 2, 2, 3
};

//
// Timer interrupt service routine
//
void interrupt isr(void)
{
    TMR0H = TIMER_H;
    TMR0L = TIMER_L;
    BUZZER = ~BUZZER;
    INTCONbits.TMR0IF= 0;

}

//
// This function generates millisecond delays
//
void Delay_Ms(unsigned int s)
{
    unsigned int j;
    for(j = 0; j < s; j++)__delay_ms(1);
}

//
// This function plays a note with the specified frequency and duration
//
void Sound_Play(unsigned int freq, unsigned int duration)
{
  float period;
  period = 500000.0 / freq;
  period = 65536 - period;
  TIMER_H = (char)(period / 256);
  TIMER_L = (char)(period - 256*TIMER_H);
  TMR0H = TIMER_H;
  TMR0L = TIMER_L;
  T0CONbits.TMR0ON = 1;
  Delay_Ms(duration);
  T0CONbits.TMR0ON = 0;
}

//
// This function creates seconds delay. The argument specifies the delay time in seconds
//
void Delay_Seconds(unsigned char s)
{
    unsigned char i,j;

    for(j = 0; j < s; j++)
    {
        for(i = 0; i <  100; i++)__delay_ms(10);
    }
}




void main()
{
     unsigned char i;

     ANSELC = 0;                                   	// Configure PORTC as digital
     TRISC = 0;						// Configure PORTC as output
     BUZZER = 0;					// Buzzer=0 to start with
//
// Configure TIMER0 for 16 bit, prescaler=2
//
     T0CONbits.TMR0ON = 0;              		// Timer OFF
     T0CONbits.T08BIT = 0;              		// Timer in 16 bit mode
     T0CONbits.T0CS = 0;                		// Use internal clock
     T0CONbits.T0SE = 0;                		// Low-to-high transition
     T0CONbits.PSA = 0;                 		// Use the prescaler
     T0CONbits.T0PS = 0;                		// Prescaler = 2
     
     INTCON = 0xA0;                      		// Enable global and TMR0 interrupts

     for(;;)                                   		// Endless loop
     {
       for(i = 0; i < Max_Notes; i++)             	// Do for all notes
       {
          Sound_Play(Notes[i], 400*Durations[i]);     	// Play the notes
          Delay_Ms(100);                               	// Gap between notes
       }
       Delay_Seconds(3);                                 // Repeat after 3 seconds
     }
}
